package abbot.editor;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Window;

import javax.swing.JLabel;
import javax.swing.JWindow;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.SoftBevelBorder;

import abbot.Platform;
import abbot.i18n.Strings;

/** Simple splash screen for the script editor. */
public class Costello {

    private static class SplashScreen extends JWindow {
        boolean disposeOK = false;
        public SplashScreen() { }
        public void hide() {
            if (disposeOK) {
                super.hide();
            }
        }
        public void dispose() {
            if (disposeOK) {
                super.dispose();
            }
        }
    }

    private static SplashScreen splash = null;
    public static Window getSplashScreen() {
        return splash;
    }

    /** Note that this "main" is typical of many Swing apps, in that it does
        window showing and disposal off the main thread.  Running the editor 
        under itself should provide a reasonable test for handling that
        scenario.
    */
    public static void main(String[] args) {

        if (Platform.getJavaVersionNumber() < Platform.JAVA_1_4) {
            // Mac OSX setup stuff
            System.setProperty("com.apple.mrj.application.growbox.intrudes",
                               "true");
            System.setProperty("com.apple.mrj.application.apple.menu.about.name",
                               "Costello");
            System.setProperty("com.apple.macos.use-file-dialog-packages", "true");
            System.setProperty("com.apple.macos.useScreenMenuBar", "true");
        }
        else {
            System.setProperty("apple.laf.useScreenMenuBar", "true");
            System.setProperty("com.apple.mrj.application.apple.menu.about.name",
                               "Costello");
            System.setProperty("apple.awt.showGrowBox", "true");
        }

        splash = new SplashScreen();
        // Load i18n-specific text for the splash screen
        String html = Strings.get("Splash");
        JLabel label = new JLabel(html);
        if (System.getProperty("mrj.version") == null) {
            label.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
        }
        else {
            label.setBorder(new EmptyBorder(4,4,4,4));
        }
        splash.getContentPane().add(label);
        splash.pack();
        Dimension d = splash.getToolkit().getScreenSize();
        Dimension size = splash.getSize();
        Point loc = new Point((d.width - size.width) / 2,
                              (d.height - size.height) / 2);
        splash.setLocation(loc);
        splash.show();

        try {
            ScriptEditor.main(args);
        }
        catch(Throwable e) {
            System.err.println("Unexpected exception trying to launch "
                               + "the script editor");
            e.printStackTrace();
            System.exit(1);
        }
        finally {
            splash.disposeOK = true;
            // NOTE: disposal in main still screws us up, because the dispose
            // does an invokeAndWait...
            java.awt.EventQueue.invokeLater(new Runnable() {
                public void run() { splash.dispose(); splash = null; }
            });
        }
    }
}


