package abbot.editor.editors;

import javax.swing.JComboBox;
import javax.swing.JTextField;

import abbot.Log;
import abbot.Resolver;
import abbot.tester.awt.ComponentTester;
import abbot.script.Event;

/** Edit a raw AWTEvent. */

public class EventEditor extends StepEditor {

    private Event event;
    JComboBox type;
    JComboBox kind;
    JComboBox cref;
    private boolean ignoreCombo = false;

    JTextField xValue;
    JTextField yValue;

    JTextField keyCode;

    public EventEditor(Event event) {
        super(event);
        this.event = event;
        String idtype = event.getType();
        String idkind = event.getKind();
        String[] typeValues = { "Mouse Event", "Key Event" };
        ignoreCombo = true;
        type = addComboBox("Type", typeValues[0], typeValues);
        type.setEnabled(false);
        type.setEditable(false);
        Resolver resolver = event.getResolver();
        String refid = event.getComponentID();
        if ("MouseEvent".equals(idtype)) {
            String[] kindValues = { "MOUSE_MOVED" };
            kind = addComboBox("Kind", kindValues[0], kindValues);
            kind.setEditable(false);
            kind.setEnabled(false);
            cref = addComponentSelector("On Component", refid,
                                        resolver, false);
            xValue = addTextField("X", event.getAttribute(Event.TAG_X));
            yValue = addTextField("Y", event.getAttribute(Event.TAG_Y));
        }
        else if ("KeyEvent".equals(idtype)) {
            type.setSelectedItem(typeValues[1]);
            String[] kindValues = { "KEY_PRESSED", "KEY_RELEASED" };
            kind = addComboBox("Kind", kindValues["KEY_PRESSED".equals(idkind)
                                                  ? 0 : 1], kindValues);
            kind.setEditable(false);
            cref = addComponentSelector("On Component", refid,
                                        resolver, false);
            // FIXME make a popup w/all keycodes
            keyCode = addTextField("Key Code", event.getAttribute(event.TAG_KEYCODE));
        }
        else {
            Log.warn("Unhandled ID type: " + idtype);
        }
        ignoreCombo = false;
    }

    /** Indicate that the step data may have changed and that the editor should
     * update itself.
     */
    protected void stepChanged() {
        ignoreCombo = true;
        kind.setSelectedItem(event.getAttribute(event.TAG_KIND));
        ignoreCombo = false;
        String refid = event.getComponentID();
        cref.setSelectedItem(refid);
        if (xValue != null) {
            xValue.setText(event.getAttribute(event.TAG_X));
            yValue.setText(event.getAttribute(event.TAG_Y));
        }
        else {
            keyCode.setText(event.getAttribute(event.TAG_KEYCODE));
        }
        super.stepChanged();
    }

    public void actionPeformed(java.awt.event.ActionEvent ev) {
        Object src = ev.getSource();
        if (src == cref) {
            // ignore the change (?)
        }
        else if (src == kind) {
            if (!ignoreCombo) {
                event.setAttribute(event.TAG_KIND, (String)kind.getSelectedItem());
                fireStepChanged();
            }
        }
        else if (src == xValue) {
            try {
                int value = Integer.parseInt(xValue.getText());
                event.setAttribute(event.TAG_X, String.valueOf(value));
                fireStepChanged();
            }
            catch(NumberFormatException nfe) {
                xValue.selectAll();
            }
        }
        else if (src == yValue) {
            try {
                int value = Integer.parseInt(yValue.getText());
                event.setAttribute(event.TAG_Y, String.valueOf(value));
                fireStepChanged();
            }
            catch(NumberFormatException nfe) {
                yValue.selectAll();
            }
        }
        else if (src == keyCode) {
            try {
                String codestr = keyCode.getText().trim();
                int code = ComponentTester.getKeyCode(codestr);
                event.setAttribute(event.TAG_KEYCODE, codestr);
            }
            catch(IllegalArgumentException iae) {
                keyCode.selectAll();
            }
        }
        else {
            super.actionPerformed(ev);
        }
    }

}
