package abbot.editor.recorder;

import java.awt.AWTEvent;
import java.awt.event.ComponentEvent;

import javax.swing.JInternalFrame;

import abbot.Resolver;
import abbot.script.Assert;
import abbot.script.ComponentReference;
import abbot.script.Step;

/**
 * Handle the recording of events related to an internal frame
 * (JInternalFrame). Like instances of Window, we must insert
 * waits for the showing and hiding of internal frames.
 * <P>
 * @author pickens
 * TO DO: record movement and resizing.
 */
public class JInternalFrameRecorder extends JComponentRecorder {
    private JInternalFrame frame;
    private boolean isClose;

    public static final int SE_INTERNAL_FRAME = SE_ACTION_MAP + 1;

    /**
     * Constructor for JInternalFrameRecorder.
     * @param resolver
     */
    public JInternalFrameRecorder(Resolver resolver) {
        super(resolver);
    }

    protected void init(int rtype) {
        super.init(rtype);
        frame = null;
        isClose = false;
    }

    /**
     * @see abbot.editor.recorder.ComponentRecorder#accept(java.awt.AWTEvent)
     */
    public boolean accept(AWTEvent event) {
        int id = event.getID();
        if (event.getSource() instanceof JInternalFrame
            && (id == ComponentEvent.COMPONENT_HIDDEN
                || id == ComponentEvent.COMPONENT_SHOWN)) {
            init(SE_INTERNAL_FRAME);
            return true;
        }
        return false;
    }
    
    public boolean parse(AWTEvent event) {
        int id = event.getID();
        boolean consumed = true;
        switch(getRecordingType()) {
        case SE_INTERNAL_FRAME:
            consumed = parseInternalFrame(event);
            break;
        default:
            consumed = super.parse(event);
            break;
        }
        return consumed;
    }

    /** @see abbot.editor.recorder.ComponentRecorder#parseWindowEvent(java.awt.AWTEvent) */
    protected boolean parseInternalFrame(AWTEvent event) {
        frame = (JInternalFrame)event.getSource();
        isClose = event.getID() == ComponentEvent.COMPONENT_HIDDEN;
        setFinished(true);
        return true;
    }

    protected Step createStep() {
        Step step;
        switch(getRecordingType()) {
        case SE_INTERNAL_FRAME:
            step = createInternalFrameEvent(frame, isClose);
            break;
        default:
            step = super.createStep();
            break;
        }
        return step;
    }

    /** @see abbot.editor.recorder.ComponentRecorder#createWindowEvent(java.awt.Window,boolean) */
    protected Step createInternalFrameEvent(JInternalFrame target,
                                            boolean isClose){
        ComponentReference cr = getResolver().addComponent(frame);
        Assert step = new Assert(getResolver(), 
                                 null, 
                                 "assertComponentShowing",
                                 new String[] { cr.getID() },
                                 "true", isClose);
        step.setWait(true);
        return step;       
    }
}
