package abbot.editor.recorder;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.text.JTextComponent;

import abbot.Log;
import abbot.Resolver;
import abbot.script.Action;
import abbot.script.ComponentReference;
import abbot.script.Step;

/**
 * Record basic semantic events you might find on an JTextComponent. <p>
 */
public class JTextComponentRecorder extends JComponentRecorder {

    private JTextComponent target;
    private int startIndex;
    private int endIndex;

    public static final int SE_SELECTION = 30;

    public JTextComponentRecorder(Resolver resolver) {
        super(resolver);
    }

    protected void init(int rtype) {
        super.init(rtype);
        target = null;
        startIndex = -1;
        endIndex = -1;
    }

    /** Don't store the action "default-typed"; store the key event instead. */
    protected boolean isMappedEvent(KeyEvent ev) {
        if (super.isMappedEvent(ev)) {
            javax.swing.Action action = getAction(ev);
            return !"default-typed".equals(action.getValue(action.NAME));
        }
        return false;
    }

    /** Coalesce initial click with subsequent drags to produce a
     * selection.
     */
    protected boolean dragStarted(Component target,
                                  int x, int y,
                                  int modifiers,
                                  MouseEvent dragEvent) {
        Log.debug("Tracking text selection");
        setRecordingType(SE_DROP);
        this.target = (JTextComponent)target;
        startIndex = this.target.viewToModel(new Point(x, y));
        // Concatenate drag/release with the original click
        return true;
    }

    protected boolean parseDrop(AWTEvent event) {
        boolean consumed = true;
        if (event.getID() == MouseEvent.MOUSE_DRAGGED) {
            endIndex = target.viewToModel(((MouseEvent)event).getPoint());
        }
        else if (event.getID() == MouseEvent.MOUSE_RELEASED) {
            endIndex = target.viewToModel(((MouseEvent)event).getPoint());
            setFinished(true);
        }
        else {
            consumed = super.parseDrop(event);
        }
        return consumed;
    }

    protected Step createStep() {
        Step step;
        if (getRecordingType() == SE_DROP) {
            step = createDrop(target, startIndex, endIndex);
        }
        else {
            step = super.createStep();
        }
        return step;
    }

    /** The text component click should click on the text index instead of a
        mouse coordinate. */
    // FIXME
    //protected Step createClick(Component comp, int x, int y, int mods, int count);

    protected Step createDrop(Component comp, int start, int end) {
        ComponentReference cr = getResolver().addComponent(comp);
        JTextComponent tc = (JTextComponent)comp;
        return new Action(getResolver(), null,
                          "actionSelect", new String[] {
                              cr.getID(),
                              String.valueOf(start),
                              String.valueOf(end)
                          }, JTextComponent.class);
    }
}

