package abbot.tester.awt;

import java.awt.Component;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JList;

public class JComboBoxTester extends JComponentTester {

    private JListTester listTester = new JListTester();

    /** Return an array of strings that represent the combo box list. 
     * Note that the current selection might not be included, since it's
     * possible to have a custom entry that is not included in the default
     * contents there.
     */
    public String[] getContents(JComboBox cb) {
        ArrayList list = new ArrayList();
        for (int i=0;i < cb.getItemCount();i++) {
            list.add(cb.getItemAt(i).toString());
        }
        return (String[])list.toArray(new String[list.size()]);
    }

    public void actionSelectIndex(Component comp, final int index) {
        final JComboBox cb = (JComboBox)comp;
        int current = cb.getSelectedIndex();
        if (current == index)
            return;

        if (getEventMode() == EM_PROG) {
            invokeAndWait(new Runnable() {
                public void run() {
                    cb.setSelectedIndex(index);
                }
            });
        }
        else {
            // activate it
            // NOTE: if the index is out of range, the selected item will be
            // at one end of the list.
            click(cb);
            JList list = findComboList(cb);
            listTester.actionSelectIndex(list, index);
        }
    }

    /** Find the JList in the popup raised by this combo box. */
    private JList findComboList(JComboBox cb) {
        Component comp = getFinder().findActivePopupMenu(cb);
        if (comp == null) {
            long now = System.currentTimeMillis();
            while ((comp = getFinder().findActivePopupMenu(cb)) == null) {
                if (System.currentTimeMillis() - now > 2000)
                    throw new ActionFailedException("No combo box popup found");
                sleep();
            }
        }
        
        while (!(comp instanceof javax.swing.JList)) {
            Component[] children =
                getFinder().getComponents((java.awt.Container)comp);
            for (int i=0;i < children.length;i++) {
                if (children[i] instanceof java.awt.Container) {
                    comp = children[i];
                    break;
                }
                else if (children[i] instanceof JList) {
                    comp = children[i];
                    break;
                }
            }
        }
        return (JList)comp;
    }

    public void actionSelectItem(Component comp, String item) {
        JComboBox cb = (JComboBox)comp;
        Object obj = cb.getSelectedItem();
        if (obj != null && Comparator.stringsMatch(item, obj.toString()))
            return;

        int index = -1;
        for (int i=0;i < cb.getItemCount();i++) {
            obj = cb.getItemAt(i);
            if (Comparator.stringsMatch(item, obj.toString())) {
                actionSelectIndex(comp, i);
                return;
            }
        }
        // While actions are supposed to represent real user actions, it's
        // possible that the current environment does not match sufficiently,
        // so we need to throw an appropriate exception that can be used to
        // diagnose the problem.
        throw new ActionFailedException("Item '" + item 
                                        + "' not found in JComboBox");
    }
}
