package abbot.tester.awt;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;

import javax.accessibility.AccessibleContext;

public class WindowTester extends ContainerTester {

    /** The window's size seems as good an ID as any.  If someone has a bunch
     * of frameless windows floating about, they can come up with a better
     * ID.
     */
    public String deriveTag(Component comp) {
        String tag = null;
        AccessibleContext context = ((Window)comp).getAccessibleContext();
        tag = deriveAccessibleTag(context);
        if (tag == null || "".equals(tag)) {
            Dimension size = comp.getSize();
            tag = String.valueOf(size.width) + "x" 
                + String.valueOf(size.height);
        }
        return tag;
    }

    public void actionClose(Component w) {
        close((Window)w);
        waitForIdle();
    }

    /** Move the window to the given location. */
    public void actionMove(Component w, int screenx, int screeny) {
        move((Window)w, screenx, screeny);
        waitForIdle();
    }

    /** Move the window to the given location. */
    public void actionMoveBy(Component w, int dx, int dy) {
        moveBy((Window)w, dx, dy);
        waitForIdle();
    }

    /** Resize the given window.  Note that this will fail on frames or
     * dialogs which are not resizable.
     */
    public void actionResize(Component w, int width, int height) {
        if (((w instanceof Frame) && !((Frame)w).isResizable())
            || (w instanceof Dialog) && !((Dialog)w).isResizable())
            throw new ActionFailedException("Frame/Dialog is not resizable");
        resize((Window)w, width, height);
        waitForIdle();
    }

    /** Resize the given window.  Note that this will fail on frames or
     * dialogs which are not resizable.
     */
    public void actionResizeBy(Component w, int dx, int dy) {
        if (((w instanceof Frame) && !((Frame)w).isResizable())
            || (w instanceof Dialog) && !((Dialog)w).isResizable())
            throw new ActionFailedException("Frame/Dialog is not resizable");
        resizeBy((Window)w, dx, dy);
        waitForIdle();
    }

    /** Activate the given Window.  */
    public void actionActivate(Window w) {
        activate(w);
        waitForIdle();
    }
}
