/*
 * TO TEST FOR SURE:
 * - isOpen
 * - isClose (does a dispose() call also send a close call?  Otherwise we should check
 *   for that too...
 * - isMenuEvent - what is SWT.MenuDetect ?
 * - isKeyTyped - should we look at up and down?  key and hardkey?
 * - isClick - should we look at double clicks too?
 * - isDragDrop - is this all we need to look at?
 * - parseClick() - what to do if mousePressed and target == event.widget (as 
 *   far as clickCount is concerned?)
 * - parseClick() - we ignore double-clicks...
 * - parseClick() - is DragDetect the right event type to use?
 * - parseMenuSelection() - make sure that this logic works for menu bars as 
 *   well- primarily the showCount...
 * - parseDrop - this was basically a best-guess
 * - createWindowEvent - check to make sure that all string constants are correct,
 *   specifically the "assertFrameShowing" for the method to be called
 * - createClick - this is where you may need to do something about the String params 
 * - getAccelerator 
 */
 
 /* Parsers completed thus far (and need to be tested):
  * - parseClick
  * - parseWindowEvent
  * - parseKeyEvent 
  * - parseMenuSelection
  * - parseDrop
  */
  
  /* Parsers implemented in the original ComponentRecorder that should be
   * moved elsewhere (eventually):
   * - parseMenuSelection
   * 
   */
   // TODO Make so that we can record entire Strings instead of single keys
   
   // TODO Add post-recording processing to combine a series of actionKey() calls
   // into a single actionKeyString() call
   
package com.ibm.abbot.editor.recorder;


import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.WeakHashMap;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;

import abbot.BugReport;
import abbot.Log;

import com.ibm.abbot.script.Assert;
import com.ibm.abbot.script.Action;
import com.ibm.abbot.script.WidgetReference;
import com.ibm.abbot.tester.swt.Robot;
import com.ibm.abbot.script.Step;
import com.ibm.abbot.Resolver;
import com.ibm.abbot.tester.swt.*;

public class WidgetRecorder extends SemanticRecorder {
	private static final String[] TYPES = {
		  "any", "window", "menu", "click", "key",
		  "drag", "drop", "text", "input method"
	  };

	  /** Mappings for special keys. */
	  private static java.util.HashMap specialMap;

	  // For windows
	  private Widget window = null;
	  private boolean isClose = false;
	  // For key presses
	  private boolean modifierTyped =false;
	  private char keychar = KeyEvent.CHAR_UNDEFINED;
	  private int modifiers = 0;
	  private int keyCode = 0;
	  private String keyString = "";
	  // For clicks
	  private Widget target = null;
	  private Widget forwardedTarget = null;
	  private int x, y;
	  private boolean expectNoClick = false; // unused
	  private boolean released = false;
	  private int clickCount = 0;
	  private boolean selected = false;
	  
	  // For menu events
	  private Widget invoker = null;
	  private int menux, menuy;
	  private int showCount = 0;// the current level of the opened cascade menu
	  private Widget selectedMenuItem = null;
	  private boolean waitingForSelection = false;// all menus have been exited, so if
	  											  // we don't recive a selection or something else
	  											  // notable, selection didn't occur
	  
	  private MenuItem awtMenuTarget = null;
	  //private Frame awtMenuFrame = null;
	  //private Component menuTarget = null;
	  private boolean isPopup = false;
	  private boolean menuCanceled = false;
	  // For drag events
	  // This class is responsible for handling drag/drop once the action has
	  // been recognized by a derived class
	  private Widget dragSource = null;
	  private int dragx, dragy;
	  // For drop events
	  private Widget dropTarget = null;
	  private int dropx, dropy;
	  private boolean desktopDrop = false;
	  // misc tracking
	  private boolean mouseLeft = false;
	  // InputMethod
	  private ArrayList imKeyCodes = new ArrayList();
	  private StringBuffer imText = new StringBuffer();
	  private int imModifiers = 0;
	  /** Keep a short-term memory of windows we've seen open/close already. */
	  private static WeakHashMap closedWindows = new WeakHashMap();
	  private static WeakHashMap openedWindows = new WeakHashMap();
	
	
	/** Create a WidgetRecorder for use in capturing the semantics of a GUI
	 * action.
	 */ 
	public WidgetRecorder(Resolver resolver) {
		super(resolver);
	}	

	/** Does the given event indicate a window was shown? */
	private boolean isOpen(Event event) {
		int type = event.type;
		Widget widget = event.widget;
		
		return(	widget!=null &&
				widget instanceof Decorations &&
				type==SWT.Show );			
	}	
	
	/** Does the given event indicate a window was closed? */
	private boolean isClose(Event event) {
		int type = event.type;
		Widget widget = event.widget;
		
		return (widget!=null &&
				widget instanceof Decorations &&
				type==SWT.Close);
	}	

	/** Returns whether this WidgetRecorder wishes to accept the given
	 * event.  If the event is accepted, the recorder must invoke init() with
	 * the appropriate semantic event type.
	 */
	public boolean accept(Event event) {
		int rtype = SE_NONE;

		if (isWindowEvent(event)) {
			rtype = SE_WINDOW;
		}
		else if (isMenuEvent(event)) {
			rtype = SE_MENU;
		}
		// For now, only handle individual key strokes
		else if (isKeyTyped(event)) {
			rtype = SE_KEY;
		}
		else if (isClick(event)) {
			rtype = SE_CLICK;
		}
		else if (isDragDrop(event)) {
			rtype = SE_DROP;
		}
		else {
			//Log.debug("Ignoring " + Robot.toString(event));
			Log.debug("Ignoring "+event);
		}

		init(rtype);
		boolean accepted = rtype != SE_NONE;
		if (accepted)
			//Log.debug("Accepted " + WidgetTester.toString(event));
			Log.debug("Accepted "+ event);
		return accepted;
	}


	/** Test whether the given event is a trigger for a window event. 
	 */
	protected boolean isWindowEvent(Event event) {
		int type = event.type;
		Widget widget = event.widget;
		
		return (widget!=null &&
				widget instanceof Decorations &&
				(type==SWT.Close || type==SWT.Show));
	}

	/** Test whether the given event is for a menu.
	 */
	protected boolean isMenuEvent(Event event) {
		int type = event.type;
		Widget widget = event.widget;
		
		return ((widget!=null &&
				widget instanceof Menu) ||
				type==SWT.MenuDetect ||
				widget instanceof MenuItem);
	}

	/** Test whether the given event is a trigger for a key typed.
	 */
	protected boolean isKeyTyped(Event event) {
		int type = event.type;
		return ( type==SWT.KeyDown);//||
				 //type==SWT.KeyUp ||
				 //type==SWT.HardKeyUp) ||
				 //type==SWT.HardKeyDown);
				 
	}

	/** Test whether the given event is a trigger for a mouse button click.
	 */ 
	protected boolean isClick(Event event) {
		int type = event.type;
		int button = event.button;
		return ( 	(type==SWT.MouseDown || 
					type==SWT.MouseUp ||
					type==SWT.MouseDoubleClick) &&
				 	button==1);
	}

	/** Test whether the given event precurses a drop. */
	protected boolean isDragDrop(Event event) {
		return event.type == SWT.DragDetect;
	}

	/** Default to recording a drag if it looks like one. */
	// FIXME may be some better detection, like checking for DND interfaces. */
	protected boolean canDrag() {
		return true;
	}

	/** Default to waiting for multiple clicks. */
	protected boolean canMultipleClick() {
		return true;
	}

	/** parses a button-1 click **/
	boolean foundDoubleClick = false;
	protected boolean parseClick(Event event){
		boolean consumed = true;
		int type = event.type;
		if(type==SWT.MouseDown){
			Log.debug("Parsing mouse down");
			target = event.widget;
			// Add the reference now, to avoid potential problems creating
			// it later.  If the step is canceled, we might wind up with
			// an unused saved reference, but we can deal with that
			// somewhere else. 
			getResolver().addWidget(target);
			x = event.x;
			y = event.y;
			modifiers = event.stateMask | SWT.BUTTON1;
			clickCount = 1;
			released = false;			
			
		}
		else if (type == SWT.MouseUp) {
			Log.debug("Parsing mouse up");
			target = event.widget;
			released = true;
//			// Optionally allow for multiple clicks
//			if (!canMultipleClick())
//				setFinished(true);
		}		
		else if (type == SWT.MouseExit) {
			if (event.widget == target && forwardedTarget == null)
				mouseLeft = true;
		}
		else if (type == SWT.MouseEnter) {
			if (event.widget == target && forwardedTarget == null)
				mouseLeft = false;
		}
		else if (type == SWT.MouseDoubleClick){
			target = event.widget;
			released = true;
			clickCount = 2;
			setFinished(true);	
			foundDoubleClick = true;
		}
		else if (type==SWT.DragDetect && canDrag()) {
			Log.debug("Changing click to drag start");
			// Was actually a drag; pass off to drag handler
			setRecordingType(SE_DRAG);
			consumed = dragStarted(target, x, y, modifiers, event);
		}
		else if(type==SWT.MouseHover||type==SWT.MouseMove||type==SWT.Paint){
					// do nothing		
		}
		else if(type==SWT.Selection || type==SWT.DefaultSelection){
			selected = true;
			setFinished(true);
			/* TODO Actually use the selected flag in the various testers
			 * so that actionClick will call actionSelect if the click fails
			 * to select for some reason
			 */
	
		}
		else {
			// All other events should cause the click to finish,
			// but don't register a click unless we've received the release
			// event. 
			if (released) {
				consumed = false;
				setFinished(true);
			}
		}
				
		return consumed;	
	}

	protected boolean parseWindowEvent(Event event) {
		boolean consumed = true;
		isClose = isClose(event);
		// Keep track of window open/close state so we don't parse the same
		// semantic event twice (e.g. COMPONENT_SHOWN + WINDOW_OPENED or
		// multiple WINDOW_CLOSED events).
		if (isClose) {
			closedWindows.put(event.widget, event.widget);
			openedWindows.remove(event.widget);
			Log.log("close=" + event);
		}
		else {
			openedWindows.put(event.widget, event.widget);
			closedWindows.remove(event.widget);
		}
		
		window = event.widget;
		setFinished(true);
		return consumed;
	}

	protected boolean parseKeyEvent(Event event) {
		int type = event.type; 
		boolean consumed = true;
		
		if(type==SWT.KeyDown){
			target = event.widget;
			keychar = event.character;
			keyCode = event.keyCode;
			modifiers = event.stateMask;
			
			// ignore all key events where the modifier is the key typed, and
			// all events where the modifier mask is other than a simple shift
			if( !Character.isDefined(keychar) || 
				(keyCode & SWT.MODIFIER_MASK)!=0 ||
				((modifiers& SWT.MODIFIER_MASK)!=0 && modifiers!=SWT.SHIFT)){
				setFinished(true);
				setRecordingType(SE_NONE);				
				}
			else{
				setFinished(true);
				modifierTyped = false;
			}
		}
		else{
			setFinished(true);
			consumed = false;
			if(modifierTyped)
				setRecordingType(SE_NONE);
		}		
		
		return consumed;	
	}

	protected boolean parseMenuSelection(Event event) {
		if(event.type==SWT.Selection){
			System.out.print("");
		}
		int type = event.type;
		boolean consumed = true;
		if(type==SWT.MenuDetect){
			target = event.widget;	
			x = event.x;
			y = event.y;
		}
		else if(type==SWT.Arm){
			// do nothing- wait for a menu-show
		}
		else if(type==SWT.Selection){
			selectedMenuItem = event.widget;
			setFinished(true);
		}
		else if(type==SWT.Show){
			showCount++;	
		}
		else if (type==SWT.Hide){
			showCount--;
			if(showCount<=0){
				waitingForSelection = true;
			}
		}
		else if(type==SWT.MouseExit){
			waitingForSelection = true;
		}
		else if(waitingForSelection){
			// the selection event should be next, so if we didn't grab it
			// above, then nothing happened; 
			setFinished(true);
			consumed = false;
		}

		return consumed;
	}

	protected boolean parseDrop(Event event){
		int type = event.type;
		boolean consumed = true;
		
		if(type==SWT.MouseUp){
			dropTarget = event.widget;
			dropx = event.x;
			dropy = event.y;
			setFinished(true);
			
			// maybe set desktopDrop here...
		}
		
		return consumed;
	}

	/** Handle an event.  Return whether the event was consumed. */
	public boolean parse(Event event) {
		Log.debug("Parsing " +  event
				  + " as " + TYPES[getRecordingType()]);

		// Default handling is event consumed, and assume not finished
		boolean consumed = true;

		switch(getRecordingType()) {
		case SE_WINDOW:
			consumed = parseWindowEvent(event);
			break;
		case SE_KEY:
			consumed = parseKeyEvent(event);
			break;
		case SE_CLICK: 
			consumed = parseClick(event);
			break;
		case SE_MENU:
			consumed = parseMenuSelection(event);
			break;
		case SE_DROP:
			consumed = parseDrop(event);
			break;
		default:
			Log.warn("Unknown input type: " + getRecordingType());
			// error
			break;
		}
		if (isFinished()) {
			try {
				setStep(createStep());
				Log.log("Semantic event recorded");
			}
			catch(Throwable thr) {
				thr.printStackTrace();
				BugReport br = new BugReport("An error was encountered while "
											 + "recording; the offending step "
											 + "has been omitted.", thr);
				//setStatus("An error was encountered, see console for details");
				setRecordingType(SE_NONE);
				throw br;
			}
		}

		return consumed;
	}

	/** Returns whether the first drag motion event should be consumed.
	 * Derived classes may override this to provide custom drag behavior.
	 * Default behavior saves the drag initiation event by itself.
	 */
	protected boolean dragStarted(Widget target,
											   int x, int y,
											   int modifiers,
											   Event dragEvent) {
		Log.debug("Setting drag target to " + Robot.toString(target));
		dragSource = target;
		dragx = x;
		dragy = y;
		setFinished(true);
		return false;
	}

	/** Returns the script step generated from the events recorded so far. */
	protected Step createStep() {
		Step step = null;
		Log.debug("Creating step for semantic recorder");
		switch(getRecordingType()) {
		case SE_WINDOW:
			step = createWindowEvent(window, isClose);
			break;
		case SE_MENU: 
			if(selectedMenuItem != null){
				// TODO call createPopupMenuSelection(), using (x,y) on
				// items in popups
				step = createMenuSelection(selectedMenuItem);
			}
			break;
		case SE_KEY: {		
				step = createKey(target, getAccelerator());
			break;
		}
		case SE_CLICK: {
			if (!mouseLeft&&clickCount!=0) {
				step = createClick(target, x, y, modifiers,
								   canMultipleClick() ? clickCount : 1);
			}
			else {
				step = null;
			}
			break;
		}
		case SE_DRAG: {
			step = createDrag(dragSource, dragx, dragy);
			break;
		}
		case SE_DROP:
			if (desktopDrop) {
				// drop is to the desktop, apparently.  for now just save
				// coordinates 
				// FIXME
				step = null;
			}
			else {
				step = createDrop(dropTarget, dropx, dropy);
			}
			break;
		default:
			step = null;
			break;
		}

		return step;
	}

	/** Create a wait for the window show/hide.  Use an appropriate identifier
		string.
	*/
	protected Step createWindowEvent(Widget window, boolean isClose) {
		// Prefer name to title to tag
		String name = getFinder().getWidgetName(window);
		String title = name != null
			? name : getFinder().getWidgetDecorationsTitle(window);
		if (title == null || "".equals(title))
			title = WidgetTester.getTag(window);
		Assert step = new Assert(getResolver(), 
								 null, "assertDecorationsShowing",
								 new String[] { title }, "true", isClose);
		step.setWait(true);
		return step;
	}

	protected Step createActivate(Widget window) {
		WidgetReference ref = getResolver().addWidget(window);
		return new Action(getResolver(), 
						  null, "actionActivate",
						  new String[] { ref.getID() },
						  Decorations.class);
	}

	protected Step createMenuSelection(Widget menuItem) {
		WidgetReference cr = getResolver().addWidget(menuItem);
		Step step = new Action(getResolver(), 
							   null, "actionSelectMenuItem",
							   new String[] { cr.getID() });
		return step;
	}

	public void doNothing(){
		
	}

	protected Step createPopupMenuSelection(Widget menuItem, int x, int y){
		WidgetReference itemRef = getResolver().addWidget(menuItem);
		Step step = new Action(getResolver(), 
							   null, "actionSelectPopupMenuItem",
							   new String[] { itemRef.getID(), 
											  String.valueOf(x), 
											  String.valueOf(y) });
		return step;
	}

	protected Step createKey(Widget widget, int accelerator) {
		WidgetReference ref = getResolver().addWidget(widget);        
		return new Action(getResolver(), null,
						  "actionKey",
						  new String[] { String.valueOf(accelerator)});
	}

	protected Step createDrop(Widget widget, int x, int y) {
		WidgetReference wr = getResolver().addWidget(widget);
		Step step = new Action(getResolver(), 
							   null, "actionDrop", new String[] { 
								   wr.getID(), 
								   String.valueOf(x), 
								   String.valueOf(y)
							   });
		return step;
	}

	protected Step createDrag(Widget widget, int x, int y) {
		WidgetReference wr = getResolver().addWidget(widget);
		Step step = new Action(getResolver(), 
							   null, "actionDrag", new String[] {
								   wr.getID(),
								   String.valueOf(x), String.valueOf(y)
							   });
		return step;
	}

	/** Create a click event with the given event information. */
	protected Step createClick(Widget target, int x, int y,
							   int mods, int count) {
		WidgetReference wr = getResolver().addWidget(target);
		ArrayList args = new ArrayList();
		args.add(wr.getID());
		args.add(String.valueOf(x));
		args.add(String.valueOf(y));
		args.add(String.valueOf(mods));
		args.add(String.valueOf(count));
		return new Action(getResolver(), null, "actionClick",
						  (String[])args.toArray(new String[args.size()]));
	}

	protected Step createInputMethod(ArrayList codes, String text) {
		Log.debug("Text length is " + text.length());
		return new Action(getResolver(), null,
						  "actionKeyString", 
						  new String[] { text });
	}

	protected void init(int recordingType) {
		super.init(recordingType);
		target = null;
		forwardedTarget = null;
		mouseLeft = released = expectNoClick = false;
		clickCount = 0;
		showCount = 0;
		selectedMenuItem = null;
		waitingForSelection = false;
		modifiers = 0;
		keychar = KeyEvent.CHAR_UNDEFINED;
		keyCode = 0;
		invoker = null;
		awtMenuTarget = null;
		menuCanceled = false;
		dragSource = dropTarget = null;
		desktopDrop = false;
		window = null;
		isClose = false;
		imKeyCodes.clear();
		imText.delete(0, imText.length());
		imModifiers = 0;
		modifierTyped = false;
		keyString = "";
	}
	
	/* calculate the appropriate accelerator from keyCode, keychar, and modifiers **/
	protected int getAccelerator(){
		int accel = 0;
		
		if(keyCode!=0){
			accel |= SWT.KEYCODE_BIT;
			accel |= keyCode;	
			accel |= modifiers;
		}		
		else	
			accel |= keychar;
				
		return accel;
	}
}

