package com.ibm.abbot.tester.swt; /* TAKEN DIRECTLY FROM abbot.tester */

import java.lang.reflect.Array;

import abbot.*;

/** Utility class to perform comparisons. */
public class Comparator {
    private Comparator() { }

    /** Match with a regexp if the pattern contains a ".*" or is bounded by
     * slashes (/regexp/).  Requiring exact matches eliminates the need to
     * always include the start "^" and finish "$" symbols.
     */
    public static boolean stringsMatch(String pattern, String actual) {
        Log.debug("Comparing expected " + pattern + " against " + actual);
        if (pattern.startsWith("/") && pattern.endsWith("/")) {
            pattern = pattern.substring(1, pattern.length()-1);
            Log.debug("Regexp matching literal <"
                      + actual + "> against <" + pattern + ">");
            return Regexp.stringMatch(pattern, actual);
        }
        else if (pattern.indexOf(".*") != -1) {
            Log.debug("Regexp matching literal <" 
                      + actual + "> against <" + pattern + ">");
            return Regexp.stringMatch(pattern, actual);
        }
        else {
            return pattern.equals(actual);
        }
    }

    /** Perform piecewise comparisons of arrays in addition to regular
        comparisons.  */
    public static boolean equals(Object obj1, Object obj2) {
        Log.debug("Comparing " + obj1 + " against " + obj2);
        boolean result = false;
        if (obj1 == null && obj2 == null)
            result = true;
        else if (obj1 != null && obj1.equals(obj2))
            result = true;
        // If both are strings, check for a regexp match
        else if (obj1 instanceof String && obj2 instanceof String
            && stringsMatch((String)obj1, (String)obj2))
            result = true;

        else if (obj1.getClass().isArray() && obj2.getClass().isArray()
                 && Array.getLength(obj1) == Array.getLength(obj2)) {
            result = true;
            for (int i=0;i < Array.getLength(obj1);i++) {
                if (!equals(Array.get(obj1, i), Array.get(obj2, i))) {
                    result = false;
                    break;
                }
            }
        }
        Log.debug("Compares " + result);
        return result;
    }

}
