
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Control.
 */
public class ControlTester extends WidgetTester{
	/* Widget properties that are obtainable by member getter functions */
	protected Color background, foreground;
	protected int borderWidth;
	protected Rectangle bounds;
	protected boolean enabled, visible;
	protected Font font;
	protected Object layoutData;
	protected Point location,size;
	protected Menu menu;
	protected Composite parent;
	protected Shell shell;
	protected String toolTipText;
	
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */
	protected synchronized void getProperties(final Control c){
		super.getProperties(c);		
		Robot.syncExec(c.getDisplay(),this,new Runnable(){
			public void run(){
				background = c.getBackground();
				foreground = c.getForeground();
				borderWidth = c.getBorderWidth();
				bounds = c.getBounds();
				enabled = c.getEnabled();
				visible = c.getVisible();
				font = c.getFont();
				layoutData = c.getLayoutData();
				location = c.getLocation();
				size = c.getSize();
				menu = c.getMenu();
				parent = c.getParent();
				shell = c.getShell();
				toolTipText = c.getToolTipText();
			}			
		});		
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized Color getBackground(Control c) {
		getProperties(c);
		return background;
	}


	public synchronized int getBorderWidth(Control c) {
		getProperties(c);
		return borderWidth;
	}

	public synchronized Rectangle getBounds(Control c) {
		getProperties(c);
		return bounds;
	}

	public synchronized boolean isEnabled(Control c) {
		getProperties(c);
		return enabled;
	}

	public synchronized Font getFont(Control c) {
		getProperties(c);
		return font;
	}

	public synchronized Color getForeground(Control c) {
		getProperties(c);
		return foreground;
	}

	public synchronized Object getLayoutData(Control c) {
		getProperties(c);
		return layoutData;
	}

	public synchronized Point getLocation(Control c) {
		getProperties(c);
		return location;
	}

	public synchronized Menu getMenu(Control c) {
		getProperties(c);
		return menu;
	}

	public synchronized Composite getParent(Control c) {
		getProperties(c);
		return parent;
	}

	public synchronized Shell getShell(Control c) {
		getProperties(c);
		return shell;
	}

	public synchronized Point getSize(Control c) {
		getProperties(c);
		return size;
	}

	public synchronized String getToolTipText(Control c) {
		getProperties(c);
		return toolTipText;
	}

	public synchronized boolean isVisible(Control c) {
		getProperties(c);
		return visible;
	}
	/* End getters */
}
