
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Decorations.
 */
public class DecorationsTester extends CanvasTester{
	/* Widget properties that are obtainable by member getter functions */
	protected Rectangle bounds;
	protected Rectangle clientArea;
	protected Button defaultButton;
	protected Image image;
	protected Point location,size;
	protected String text;
	protected boolean maximized, minimized;
	protected Menu menuBar;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Decorations d){
		super.getProperties(d);
		Robot.syncExec(d.getDisplay(),this,new Runnable(){
			public void run(){
				bounds = d.getBounds();
				clientArea = d.getClientArea();
				defaultButton = d.getDefaultButton();
				image = d.getImage();
				location = d.getLocation();
				size = d.getSize();
				text = d.getText();
				maximized = d.getMaximized();
				minimized = d.getMinimized();
				menuBar = d.getMenuBar();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized Rectangle getBounds(Decorations d) {
		getProperties(d);
		return bounds;
	}

	public synchronized Rectangle getClientArea(Decorations d) {
		getProperties(d);
		return clientArea;
	}

	public synchronized Button getDefaultButton(Decorations d) {
		getProperties(d);
		return defaultButton;
	}

	public synchronized Image getImage(Decorations d) {
		getProperties(d);
		return image;
	}

	public synchronized Point getLocation(Decorations d) {
		getProperties(d);
		return location;
	}

	public synchronized boolean getMaximized(Decorations d) {
		getProperties(d);
		return maximized;
	}

	public synchronized Menu getMenuBar(Decorations d) {
		getProperties(d);
		return menuBar;
	}

	public synchronized boolean getMinimized(Decorations d) {
		getProperties(d);
		return minimized;
	}

	public synchronized Point getSize(Decorations d) {
		getProperties(d);
		return size;
	}

	public synchronized String getText(Decorations d) {
		getProperties(d);
		return text;
	}
	/* End getters */
}
