
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Scrollable.
 */
public class ScrollableTester extends ControlTester {
	/* Widget properties that are obtainable by member getter functions */
	protected Rectangle clientArea;
	protected ScrollBar horizontalBar;
	protected ScrollBar verticalBar;
	
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Scrollable s){
		super.getProperties(s);
		Robot.syncExec(s.getDisplay(),this,new Runnable(){
			public void run(){
				clientArea = s.getClientArea();
				horizontalBar = s.getHorizontalBar();
				verticalBar = s.getVerticalBar();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized Rectangle getClientArea(Scrollable s) {
		getProperties(s);
		return clientArea;
	}

	public synchronized ScrollBar getHorizontalBar(Scrollable s) {
		getProperties(s);
		return horizontalBar;
	}

	public synchronized ScrollBar getVerticalBar(Scrollable s) {
		getProperties(s);
		return verticalBar;
	}
	/* End getters */

}
