
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Shell.
 */
public class ShellTester extends DecorationsTester {
	/* Widget properties that are obtainable by member getter functions */
	protected Rectangle bounds;
	protected boolean enabled;
	protected int imeInputMode;
	protected Point location;
	protected Shell shell;
	protected Shell[] shells;
	
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Shell s){
		super.getProperties(s);
		Robot.syncExec(s.getDisplay(),this,new Runnable(){
			public void run(){
				bounds = s.getBounds();
				enabled = s.getEnabled();
				imeInputMode = s.getImeInputMode();
				location = s.getLocation();
				shell = s.getShell();
				shells = s.getShells();
			}
		});		
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized Rectangle getBounds(Shell s) {
		getProperties(s);
		return bounds;
	}

	public synchronized boolean isEnabled(Shell s) {
		getProperties(s);
		return enabled;
	}

	public synchronized int getImeInputMode(Shell s) {
		getProperties(s);
		return imeInputMode;
	}

	public synchronized Point getLocation(Shell s) {
		getProperties(s);
		return location;
	}

	public synchronized Shell getShell(Shell s) {
		getProperties(s);
		return shell;
	}

	public synchronized Shell[] getShells(Shell s) {
		getProperties(s);
		return shells;
	}
	/* End getters */
}
