
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type TabFolder.
 */
public class TabFolderTester extends CompositeTester{
	/* Widget properties that are obtainable by member getter functions */
	protected Rectangle clientArea;
	protected TabItem item;
	protected int itemCount;
	protected TabItem[] items;
	protected TabItem[] selection;
	protected int selectionIndex;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final TabFolder folder){
		super.getProperties(folder);
		Robot.syncExec(folder.getDisplay(),this,new Runnable(){
			public void run(){
				clientArea = folder.getClientArea();
				itemCount = folder.getItemCount();
				items = folder.getItems();
				selection = folder.getSelection();
				selectionIndex = folder.getSelectionIndex();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized TabItem getItem(final TabFolder folder,final int index){
		Robot.syncExec(folder.getDisplay(),this,new Runnable(){
			public void run(){
				item = folder.getItem(index);
			}
		});		
		return item;
	}

	public synchronized Rectangle getClientArea(TabFolder folder) {
		getProperties(folder);
		return clientArea;
	}

	
	public synchronized int getItemCount(TabFolder folder) {
		getProperties(folder);
		return itemCount;
	}

	
	public synchronized TabItem[] getItems(TabFolder folder) {
		getProperties(folder);
		return items;
	}

	
	public synchronized TabItem[] getSelection(TabFolder folder) {
		getProperties(folder);
		return selection;
	}

	
	public synchronized int getSelectionIndex(TabFolder folder) {
		getProperties(folder);
		return selectionIndex;
	}
	/* End getters */
}
