
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type ToolBar.
 */
public class ToolBarTester extends CompositeTester{
	/* Widget properties that are obtainable by member getter functions */
	protected ToolItem item;
	protected int itemCount;
	protected ToolItem[] items;
	protected int rowCount;
	
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final ToolBar bar){
		super.getProperties(bar);
		Robot.syncExec(bar.getDisplay(),this,new Runnable(){
			public void run(){
				itemCount = bar.getItemCount();
				items = bar.getItems();
				rowCount = bar.getRowCount();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized ToolItem getItem(final ToolBar bar, final int index){
		Robot.syncExec(bar.getDisplay(),this,new Runnable(){
			public void run(){
				item = bar.getItem(index);
			}
		});	
		return item;
	}
	
	public synchronized ToolItem getItem(final ToolBar bar, final Point point){
		Robot.syncExec(bar.getDisplay(),this,new Runnable(){
			public void run(){
				item = bar.getItem(point);
			}
		});	
		return item;
	}
	
	public synchronized int getItemCount(ToolBar bar){
		getProperties(bar);
		return itemCount;		
	}

	public synchronized ToolItem[] getItems(ToolBar bar){
		getProperties(bar);
		return items;		
	}	
	
	public synchronized int getRowCount(ToolBar bar){
		getProperties(bar);
		return rowCount;
	}
	/* End getters */
}
