
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Tree.
 */
public class TreeTester extends CompositeTester {
	/* Widget properties that are obtainable by member getter functions */
	protected TreeItem item, parentItem, topItem;
	protected int itemCount, itemHeight, selectionCount;
	protected TreeItem[] items,selection;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Tree tree){
		Robot.syncExec(tree.getDisplay(),this,new Runnable(){
			public void run(){
				parentItem = tree.getParentItem();
				topItem = tree.getTopItem();
				itemCount = tree.getItemCount();
				itemHeight = tree.getItemHeight();
				selectionCount = tree.getSelectionCount();
				items = tree.getItems();
				selection = tree.getSelection();
			}			
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized TreeItem getItem(final Tree tree, final Point point){
		Robot.syncExec(tree.getDisplay(),this,new Runnable(){
			public void run(){
				item = tree.getItem(point);
			}			
		});
		return item;		
	}

	public synchronized int getItemCount(Tree tree) {
		getProperties(tree);
		return itemCount;
	}

	public synchronized int getItemHeight(Tree tree) {
		getProperties(tree);
		return itemHeight;
	}

	public synchronized TreeItem[] getItems(Tree tree) {
		getProperties(tree);
		return items;
	}

	public synchronized TreeItem getParentItem(Tree tree) {
		getProperties(tree);
		return parentItem;
	}

	public synchronized TreeItem[] getSelection(Tree tree) {
		getProperties(tree);
		return selection;
	}

	public synchronized int getSelectionCount(Tree tree) {
		getProperties(tree);
		return selectionCount;
	}

	public synchronized TreeItem getTopItem(Tree tree) {
		getProperties(tree);
		return topItem;
	}
	/* End getters */
}
