package example;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;

public class MyCode {

    private static class PopupAdapter extends MouseAdapter {
        private boolean dynamic;
        private JPopupMenu cachedMenu = null;
        private int invokes = 0;
        public PopupAdapter(boolean dynamic) {
            this.dynamic = dynamic;
        }
        /** Some platforms popup here... */
        public void mousePressed(MouseEvent ev) {
            maybePopup(ev);
        }
        /** And some platforms popup here... */
        public void mouseReleased(MouseEvent ev) {
            maybePopup(ev);
        }
        /** And just in case... */
        public void mouseClicked(MouseEvent ev) {
            maybePopup(ev);
        }
        private void maybePopup(MouseEvent ev) {
            if (ev.isPopupTrigger()) {
                JPopupMenu menu = getPopupMenu();
                menu.pack();
                menu.show((Component)ev.getSource(), ev.getX(), ev.getY());
            }
        }
        private JPopupMenu getPopupMenu() {
            JPopupMenu menu = cachedMenu;
            if (menu == null) {
                menu = new JPopupMenu();
                if (dynamic) {
                    menu.add(new JMenuItem("Invoked " + ++invokes + " times"));
                    menu.add(new JSeparator());
                }
                menu.add(new JMenuItem("Black"));
                menu.add(new JMenuItem("Blue"));
                menu.add(new JMenuItem("Orange"));
                JMenu submenu = new JMenu("Other");
                submenu.add(new JMenuItem("White"));
                submenu.add(new JMenuItem("Green"));
                menu.add(submenu);
                if (!dynamic)
                    cachedMenu = menu;
            }
            return menu;
        }
    }

    public static void main(String[] args) {

        final JFrame frame = new JFrame("My Code");
        JPanel pane = new JPanel();
        pane.setName("My Pane");
        JLabel label = new JLabel("Static");
        label.addMouseListener(new PopupAdapter(false));
        pane.add(label);
        label = new JLabel("Dynamic");
        label.addMouseListener(new PopupAdapter(true));
        pane.add(label);
        JButton button = new JButton("Button");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                JOptionPane.showMessageDialog(frame, "My Dialog Message");
            }
        });
        pane.add(button);
        JTextField tf = new CustomTextField("Text field");
        tf.setName("My Text Field");
        pane.add(tf);
        JComboBox cb = new JComboBox();
        for (int i=0;i < 20;i++)
            cb.addItem("Combo " + i);
        pane.add(cb);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                //System.out.println("Action: " + ev.getActionCommand());
            }
        };

        String[] myListData = { "zero", "one", "two", "three",
                                "four", "five",
                                "six", "seven", "eight"};
        JList myList = new JList(myListData);
        myList.setToolTipText("This is a list");
        myList.setName("My List");
        myList.setVisibleRowCount(4);
        JScrollPane myScrollPane = new JScrollPane(myList);
        myScrollPane.setName("My ScrollPane");
        pane.add(myScrollPane);

        JTree myTree = new JTree();
        myTree.setVisibleRowCount(4);
        pane.add(new JScrollPane(myTree));

        String[][] data = new String[][] {
            { "0 one", "0 two", "0 three", "0 four" },
            { "1 one", "1 two", "1 three", "1 four" },
            { "2 one", "2 two", "2 three", "2 four" },
            { "3 one", "3 two", "3 three", "3 four" },
            { "4 one", "4 two", "4 three", "4 four" },
            { "5 one", "5 two", "5 three", "5 four" },
        };
        String[] names = { "one", "two", "three", "four" };
        JTable table = new JTable(data, names);
        table.setPreferredScrollableViewportSize(new Dimension(200, myTree.getPreferredSize().height));
        JScrollPane scroll = new JScrollPane(table);
        pane.add(scroll);

        JTextArea ta = new JTextArea("Four score and seven hundred years ago, our forebears extended claws reaching from the innermost mind to the outer limits", 10, 20);
        ta.setLineWrap(true);
        ta.setWrapStyleWord(true);
        pane.add(new JScrollPane(ta));


        frame.setContentPane(pane);
        JMenuBar menubar = new JMenuBar();
        menubar.setName("My Menu Bar");
        JMenu menu = new JMenu("File");

        JMenuItem mitem = new JMenuItem("Item 1");
        menu.add(mitem);
        JMenu submenu = new JMenu("File submenu"); 
        menu.add(submenu);
        mitem = new JMenuItem("Quit");
        menu.add(mitem);

        mitem = new JMenuItem("Submenu item");
        mitem.addActionListener(al);
        submenu.add(mitem);

        JMenu menu2 = new JMenu("Edit");
        mitem = new JMenuItem("Copy");
        menu2.add(mitem);

        menubar.add(menu);
        menubar.add(menu2);
        frame.setJMenuBar(menubar);
        frame.pack();
        frame.setSize(400, 400);
        frame.show();
    }

}

class CustomTextField extends JTextField {
    public CustomTextField(String contents) {
        super(contents);
    }
    public String getText() { 
        return super.getText();
    }
}

