package junit.extensions.abbot;

import junit.framework.TestCase;
import abbot.ComponentFinder;
import abbot.DefaultComponentFinder;
import abbot.Log;
import abbot.script.Script;
import abbot.script.StepRunner;

/** Simple wrapper for a test script to run under JUnit. */

public class ScriptFixture extends TestCase {

    private static final ComponentFinder finder = 
        DefaultComponentFinder.getFinder();
    private Script script;

    /** Construct a test case with the given name.  It is essential that the
     * name be passed to super() unmodified, or the JUnit GUI will consider it
     * a different test.
     */
    public ScriptFixture(String filename) { 
        super(filename);
        script = new Script(filename);
    }

    /** Obtain a component finder to look up components. */
    protected ComponentFinder getFinder() {
        return finder;
    }

    /** Override the default TestCase runTest method to invoke the script. */
    protected void runTest() throws Throwable {
        Log.log("Running " + script + " with " + getClass());
        StepRunner runner = new StepRunner();
        try {
            runner.run(script);
        }
        finally {
            // Don't let this test's components interfere with subsequent tests
            runner.terminate();
            Log.log(script.toString() + " finished");
        }
    }

    /** Assumes each argument is an Abbot script.  Runs each one. */
    public static void main(String[] args) {
        ScriptTestSuite.main(args);
    }
}
