package junit.extensions.abbot;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import abbot.Log;

/** Provides automatic test suite generation given command-line arguments.
 * Also allows for a single test to be run if it is specified.
 */

public class TestHelper {
    private TestHelper() { }

    public static Test generateSuite(Class[] classes) {
        TestSuite suite = new TestSuite();
        for (int i=0;i < classes.length;i++) {
            try {
                java.lang.reflect.Method suiteMethod =
                    classes[i].getMethod("suite", null);
                suite.addTest((Test)suiteMethod.invoke(null, null));
            }
            catch(Exception exc) {
                suite.addTest(new TestSuite(classes[i]));
            }
        }
        return suite;
    }

    public static void runTests(String[] args, Class testClass) {
        args = Log.init(args);
        try {
            Test test;
            if (args.length == 1 && args[0].startsWith("test")) {
                test = (Test)testClass.getConstructor(new Class[]{
                    String.class
                }).newInstance(new Object[] { args[0] });
            }
            else {
                try {
                    test = (Test)testClass.getMethod("suite", null).
                        invoke(null, null);
                }
                catch(Exception exc) {
                    test = new TestSuite(testClass);
                }
            }
            junit.textui.TestRunner runner = new junit.textui.TestRunner();
            try {
                TestResult r = runner.doRun(test, false);
                if (!r.wasSuccessful())
                    System.exit(-1);
                System.exit(0);
            }
            catch(Throwable thr) {
                System.err.println(thr.getMessage());
                System.exit(-2);
            }
        }
        catch(Exception exc) {
            System.err.println(exc.getMessage());
            System.exit(-2);
        }
    }
}
