package abbot;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import junit.extensions.abbot.*;
import junit.extensions.abbot.Timer;
import junit.framework.TestCase;

/** Check version settings, and platform or JVM-specific bugs. */
public class PlatformTest extends ComponentTestFixture {

    public void testVersionParsing() {
        String[] versions = {
            "1.2",
            "1.2.0_01",
            "1.3",
            "1.3.1_06",
            "1.4.0_02",
            "1.4.1_01",
        };
        int[] values = { 0x1200, 0x1201, 0x1300, 0x1316, 0x1402, 0x1411 };
        for (int i=0;i < versions.length;i++) {
            int version = Platform.parse(versions[i]);
            assertEquals("Wrong version from '" + versions[i] + "'",
                         Integer.toHexString(values[i]),
                         Integer.toHexString(version));
        }
        assertTrue("Wrong version "
                   + Integer.toHexString(Platform.getJavaVersionNumber()),
                   Platform.getJavaVersionNumber() > Platform.JAVA_1_1);
    }

    private class MouseWatcher extends MouseAdapter {
        public volatile boolean gotClick = false;
        public volatile int clickCount = 0;
        public void mouseClicked(MouseEvent me) {
            gotClick = true;
            clickCount = me.getClickCount();
        }
    }

    /** This will fail if the multi-click frame bug shows up somewhere we
        don't expect it.
    */
    public void testMultiClickFrameBug() {
        String[] data = { "one", "two", "three", "four", "five" };
        JFrame frame1 = new JFrame(getName());
        JList list1 = new JList(data);
        list1.setName("List 1");
        frame1.getContentPane().add(list1);
        
        JFrame frame2 = new JFrame(getName());
        JList list2 = new JList(data);
        list2.setName("List 2");
        frame2.getContentPane().add(list2);
        
        MouseWatcher mw1 = new MouseWatcher();
        MouseWatcher mw2 = new MouseWatcher();
        list1.addMouseListener(mw1);
        list2.addMouseListener(mw2);
        showWindow(frame1);
        getRobot().click(list1);
        getRobot().waitForIdle();
        frame1.hide();
        showWindow(frame2);
        getRobot().click(list2);
        getRobot().waitForIdle();
        Timer timer = new Timer();
        while (!mw1.gotClick)
            if (timer.elapsed() > 5000)
                fail("Never received click event");
        timer.reset();
        while (!mw2.gotClick)
            if (timer.elapsed() > 5000)
                fail("Never received click event");
        if (Platform.hasMultiClickFrameBug()) {
            // This is only a warning, not a failure, since it depends on
            // platform operation and speed.
            if (mw1.clickCount == 1 && mw2.clickCount == 1) {
                Log.log("Multi-click bug expected, but wasn't there on this run");
            }
        }
        else {
            assertEquals("Multi-click should not span frames",
                         1, mw1.clickCount);
            assertEquals("Multi-click should not span frames",
                         1, mw2.clickCount);
        }
    }

    public PlatformTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, PlatformTest.class);
    }
}
