package abbot;

import junit.extensions.abbot.TestHelper;
import junit.framework.Test;
import junit.framework.TestSuite;

/** 
    Unit test suite for abbot.

    @author twall
*/

public class UnitTestSuite extends TestSuite {

    public UnitTestSuite(String name) {
        super(name);
    }

    /**
     * Assembles and returns the full Abbot test suite.
     */
    public static Test suite() {
        Class[] classes = {
            // basic framework tests
            abbot.PlatformTest.class,
            abbot.i18n.StringsTest.class,
            abbot.DefaultComponentFinderTest.class,
            abbot.script.AppClassLoaderTest.class,
            abbot.tester.awt.KeyStrokeMapTest.class,
            abbot.tester.awt.ComparatorTest.class,
            abbot.tester.awt.FileComparatorTest.class,
            abbot.tester.awt.ImageComparatorTest.class,
            // misc
            abbot.NoExitSecurityManagerTest.class,
            abbot.tester.awt.InputStateTest.class,
            // junit extensions
            junit.extensions.abbot.ScriptTestSuiteTest.class,
        };
        TestSuite suite = (TestSuite)TestHelper.generateSuite(classes);

        // Framework tests
        suite.addTest(abbot.util.AllTests.suite());
        suite.addTest(abbot.converter.AllTests.suite());

        // Script/step tests
        suite.addTest(abbot.script.AllTests.suite());

        // Component tester tests
        suite.addTest(abbot.tester.awt.TesterTestSuite.suite());

        // Recorder tests (these depend on the component testers working
        // properly). 
        suite.addTest(abbot.editor.recorder.RecorderTestSuite.suite());

        // Editor tests
        suite.addTest(abbot.editor.EditorTestSuite.suite());

        return suite;
    }

    /**
     * Runs all unit tests or simply those specified.
     */
    public static void main(String[] args) {
        TestHelper.runTests(args, UnitTestSuite.class);
    }
}
