package abbot.editor.editors;

import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.SwingUtilities;

import junit.extensions.abbot.ComponentTestFixture;
import junit.extensions.abbot.TestHelper;
import abbot.script.Assert;
import abbot.script.ComponentReference;
import abbot.script.Script;

/** Verify AssertEditor operation. */

public class AssertEditorTest extends ComponentTestFixture {

    public void testTesterMethodList() throws Throwable {
        Assert step = new Assert(new Script(), null,
                                 "assertFrameShowing", null,
                                 "true", false);
        AssertEditor editor = new AssertEditor(step);
        showFrame(editor);

        ComponentReference cr =
            new ComponentReference("MethodSelector", JComboBox.class);
        JComboBox cb = (JComboBox)getFinder().findComponent(cr);
        String[] expected = {
            "assertComponentShowing",
            "assertFrameShowing",
            "assertImage",
        };
        for (int i=0;i < cb.getItemCount();i++) {
            assertTrue("Too many combo box items (next is "
                       + cb.getItemAt(i) + ")", i < expected.length);
            assertEquals("Wrong method in combo list",
                         expected[i], cb.getItemAt(i));
        }
        assertEquals("Wrong number of combo box items",
                     expected.length, cb.getItemCount());
    }
    
    public void testComponentMethodList() throws Throwable {
        Assert step = new Assert(new Script(), null,
                                 "assertFrameShowing", null,
                                 "true", false);
        final AssertEditor editor = new AssertEditor(step);
        showFrame(editor);
        
        ComponentReference cr =
            new ComponentReference("MethodSelector", JComboBox.class);
        JComboBox cb = (JComboBox)getFinder().findComponent(cr);
        cr = step.getResolver().addComponent(cb);
        
        // Alter the step to refer to a property on a component
        step.setComponentID(cr.getID());
        step.setMethodName("getItemCount");
        SwingUtilities.invokeAndWait(new Runnable() { public void run() {
            editor.stepChanged();
        }});
        assertEquals("Wrong method selected",
                     "getItemCount", cb.getSelectedItem());
        ArrayList list = new ArrayList();
        for (int i=0;i < cb.getItemCount();i++) {
            list.add(cb.getItemAt(i));
        }
        assertEquals("Wrong target class",
                     JComboBox.class, step.getTargetClass());
        assertTrue("No items in the list", cb.getItemCount() > 0);
        String[] expected = {
            "getParent",
            "isOpaque",
            "hasFocus",
            // From the JComboBoxTester
            "getContents",
        };
        for (int i=0;i < expected.length;i++) {
            assertTrue(expected[i] + " method missing",
                       list.contains(expected[i]));
        }
    }

    /** Construct a test case with the given name. */
    public AssertEditorTest(String name) {
        super(name);
    }

    /** Run the default test suite. */
    public static void main(String[] args) {
        TestHelper.runTests(args, AssertEditorTest.class);
    }
}
