package abbot.editor.recorder;

import java.awt.AWTEvent;

import junit.extensions.abbot.*;
import junit.extensions.abbot.Timer;
import junit.framework.TestCase;
import test.EventWatcher;
import abbot.*;
import abbot.script.*;
import abbot.tester.awt.*;

/** Provide generic support for checking contents of recorded steps. */
// FIXME still needs some refactoring
public abstract class AbstractRecorderTest extends ComponentTestFixture {

    protected EventWatcher watcher;
    private Recorder recorder = null;

    public AbstractRecorderTest(String name) {
        super(name);
    }

    protected abstract Recorder getRecorder();

    protected void setUp() {
        watcher = new EventWatcher() {
            public void eventDispatched(AWTEvent event) {
                Log.log(Robot.toString(event));
                super.eventDispatched(event);
                if (recorder != null) {
                    recorder.recordEvent(event);
                }
            }
        };
    }

    protected void tearDown() {
        stopRecording();
    }

    protected void startRecording() {
        recorder = getRecorder();
        watcher.startListening(recorder.getEventMask());
    }

    protected void stopRecording() {
        watcher.stopListening();
        recorder = null;
    }

    private static final String LS = System.getProperty("line.separator");

    protected String bugInfo(String msg) {
        return "(" + new BugReport(msg) + ")";
    }

    public void assertStep(String pattern, Step step) {
        String bugInfo = bugInfo(watcher.listEvents());
        Log.log(step != null ? ("Examining step: " + step) 
                : "No step available (expected '" + pattern + "')");
        assertTrue("No step captured " + bugInfo, step != null);
        assertTrue("Incorrect step, expected <" + pattern + ">, but got <"
                   + step + "> " + bugInfo, 
                   Regexp.stringContainsMatch(pattern, step.toString()));
        // Clear the event log
        watcher.clear();
    }

    public String toString(Sequence seq) {
        String steps = "The recorded steps were the following:";
        for (int i=0;i < seq.size();i++) {
            steps += LS + seq.getStep(i);
        }
        return steps;
    }

    public void assertStepCount(int count, Sequence seq) {
        String steps = toString(seq);
        String bugInfo = bugInfo(watcher.listEvents() + LS + LS + steps);
        assertTrue("Wrong number of steps captured, expected <" + count
                   + ">, but got <" + seq.size() + "> " + bugInfo, 
                   count == seq.size());
    }
}
