package abbot.editor.recorder;

import java.awt.*;
import java.awt.event.InputEvent;
import java.util.Vector;

import javax.swing.*;

import junit.extensions.abbot.*;
import abbot.Resolver;
import abbot.Platform;
import abbot.tester.awt.*;

/**
 * Unit test to verify proper capture of user semantic events on a Choice.
 */
public class ChoiceRecorderTest 
    extends AbstractSemanticRecorderTest {

    private Frame frame;
    private Choice choice;
    private static final int MAX_ENTRIES = 20;

    public void testCaptureSelection() {
        showChoice();
        startRecording();
        ChoiceTester tester = (ChoiceTester)ComponentTester.getTester(choice);
        tester.actionSelectIndex(choice, MAX_ENTRIES - 1);
        assertStep("SelectItem(.*,item "
                   + (MAX_ENTRIES - 1) + ")");
        if (Platform.isOSX()
            && Platform.getJavaVersionNumber() < Platform.JAVA_1_4) {
            // Avoiding OSX Choice hang bug
            return;
        }

        tester.actionSelectIndex(choice, MAX_ENTRIES / 2 - 1);
        assertStep("SelectItem(.*,item "
                   + (MAX_ENTRIES/2 - 1) + ")");
        tester.actionSelectIndex(choice, 0);
        assertStep("SelectItem(.*,item 0)");
    }

    public void testESCToCancelSelection() {
        showChoice();
        startRecording();
        ComponentTester tester = ComponentTester.getTester(Component.class);
        tester.mousePress(choice);
        tester.actionKeyStroke("VK_ESCAPE");
        assertNoStep();
    }

    public void testClickToCancelSelection() {
        showChoice();
        startRecording();
        ComponentTester tester = ComponentTester.getTester(Component.class);
        tester.mousePress(choice);
        JPanel pane = (JPanel)((JFrame)frame).getContentPane();
        tester.actionClick(pane, 0, 0);
        assertNoStep();
    }

    public ChoiceRecorderTest(String name) {
        super(name);
    }

    protected SemanticRecorder createRecorder(Resolver r) {
        return new ChoiceRecorder(r);
    }

    private void showChoice() {
        choice = new Choice();
        for (int i=0;i < MAX_ENTRIES;i++) {
            choice.add("item " + i);
        }
        frame = showFrame(choice);
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, ChoiceRecorderTest.class);
    }
}
