package abbot.editor.recorder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import junit.extensions.abbot.RepeatHelper;
import abbot.*;

/**
 * Unit test to verify proper capture of user semantic events on a Window.
 */
public class JInternalFrameRecorderTest 
    extends AbstractSemanticRecorderTest {

    public void testShowHide() throws Throwable {
        if (Platform.getJavaVersionNumber() < Platform.JAVA_1_4) {
            fail("Internal frame tracking not supported prior to 1.4");
        }

        JDesktopPane dtpane = new JDesktopPane();
        final JInternalFrame frame = new JInternalFrame();
        frame.getContentPane().add(new JLabel(getName()));
        frame.addInternalFrameListener(new InternalFrameAdapter() {
        });
        // FIXME this needs to be automatically done by the recorder
        /*
        frame.addHierarchyListener(new HierarchyListener() {
            public void hierarchyChanged(HierarchyEvent he) { }
        });
        */
        dtpane.setSize(400,400);
        startRecording();
        dtpane.add(frame);
        showFrame(dtpane);
        EventQueue.invokeAndWait(new Runnable() {
            public void run() {
                frame.setLocation(10, 10);
                frame.pack();
            }
        });
        startRecording();
        EventQueue.invokeAndWait(new Runnable() {
            public void run() { frame.show(); }
        });
        assertStep("Wait for ComponentShowing");
        frame.setVisible(false);
        assertStep("Wait for !ComponentShowing");
    }

    public JInternalFrameRecorderTest(String name) {
        super(name);
    }

    protected SemanticRecorder createRecorder(Resolver r) {
        return new JInternalFrameRecorder(r);
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JInternalFrameRecorderTest.class);
    }
}
