package abbot.editor.recorder;

import java.awt.Dimension;

import javax.swing.JLabel;
import javax.swing.JTabbedPane;

import junit.extensions.abbot.RepeatHelper;
import abbot.Resolver;
import abbot.tester.awt.*;


/**
 * Unit test to verify proper capture of user semantic events on a JTabbedPane.
 */
public class JTabbedPaneRecorderTest 
    extends AbstractSemanticRecorderTest {

    private JTabbedPaneTester tester;
    private JTabbedPane tabbedPane;
    private static final int MAX_ENTRIES = 10;

    public JTabbedPaneRecorderTest(String name) {
        super(name);
    }

    protected SemanticRecorder createRecorder(Resolver r) {
        return new JTabbedPaneRecorder(r);
    }

    private void showTabbedPane() {
        tabbedPane = new JTabbedPane();
        for (int i=0;i < MAX_ENTRIES;i++) {
            tabbedPane.addTab("Tab " + i, new JLabel("This is pane #" + i) {
                public Dimension getPreferredSize() {
                    return new Dimension(300, 175);
                }
            });
        }
        tester = (JTabbedPaneTester)ComponentTester.getTester(tabbedPane);
        showFrame(tabbedPane);
    }

    public void testCaptureSelection() {
        showTabbedPane();
        startRecording();
        for (int i=MAX_ENTRIES-1;i >= 0; i--) {
            tester.actionSelectIndex(tabbedPane, i);
            assertStep("SelectTab\\(.*,Tab " + i + "\\)");
        }
    }

    /*
    public void testCaptureSelectionBug() {
        showTabbedPane();
        startRecording();
        tester.mouseMove(tabbedPane, 20, 20);
        tester.delay(1000);
        tester.mousePress(InputEvent.BUTTON1_MASK);
        tester.delay(1000);
        assertStep("SelectTab\\(.*,Tab 4\\)", false);
    }
    */

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JTabbedPaneRecorderTest.class);
    }
}
