package abbot.editor.recorder;

import javax.swing.JTable;

import junit.extensions.abbot.RepeatHelper;
import abbot.Resolver;
import abbot.tester.awt.*;

/**
 * Unit test to verify proper capture of user semantic events on a JTable.
 */
public class JTableRecorderTest 
    extends AbstractSemanticRecorderTest {

    private JTableTester tester;
    private JTable table;
    private static final int MAX_ENTRIES = 4;

    public JTableRecorderTest(String name) {
        super(name);
    }

    protected SemanticRecorder createRecorder(Resolver r) {
        return new JTableRecorder(r);
    }

    private void showTable() {
        String[][] cells = new String[MAX_ENTRIES][MAX_ENTRIES];
        for (int i=0;i < MAX_ENTRIES;i++) {
            for (int j=0;j < MAX_ENTRIES;j++) {
                cells[i][j] = "cell " + i + "," + j;
            }
        }
        String[] names = new String[MAX_ENTRIES];
        for (int i=0;i < MAX_ENTRIES;i++) {
            names[i] = "col " + i;
        }
        table = new JTable(cells, names);
        tester = (JTableTester)ComponentTester.getTester(table);
        showFrame(table);
    }

    public void testCaptureCellSelection() {
        showTable();
        startRecording();
        for (int i=MAX_ENTRIES-1;i >= 0; i--) {
            for (int j=MAX_ENTRIES-1; j >= 0;j--) {
                tester.actionSelectCell(table, i, j);
                assertStep("SelectCell\\(.*," + i + "," + j + "\\)");
            }
        }
    }

    public void testCaptureMultipleClick() {
        showTable();
        startRecording();
        int row = MAX_ENTRIES/2;
        int col = MAX_ENTRIES/2;
        tester.actionClickCell(table, row, col, "BUTTON1_MASK", 2);
        assertStep("ClickCell\\(.*," + row + "," + col + ",BUTTON1_MASK,2\\)");
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JTableRecorderTest.class);
    }
}
