package abbot.script;

import java.awt.*;
import java.awt.event.*;
import junit.extensions.abbot.*;

public class AnnotationTest
    extends ComponentTestFixture implements AWTEventListener {

    volatile private boolean opened = false;
    volatile private boolean closed = false;
    private Timer timer = new Timer();

    protected void setUp() {
        Toolkit.getDefaultToolkit().
            addAWTEventListener(this, WindowEvent.WINDOW_EVENT_MASK);
    }

    protected void tearDown() {
        Toolkit.getDefaultToolkit().removeAWTEventListener(this);
    }

    public void eventDispatched(AWTEvent event) {
        if (event.getID() == WindowEvent.WINDOW_OPENED) {
            opened = true;
            timer.reset();
        }
        else if (event.getID() == WindowEvent.WINDOW_CLOSED) {
            closed = true;
        }
    }

    private Throwable stepFailure = null;
    public void testShow() throws Throwable {
        final Annotation ann = new Annotation(new Script(), getName());
        ann.setTitle(getName());
        ann.setText("<html>This is an <b>example</b> of an annotation</html>");
        stepFailure = null;
        new Thread(getName()) { 
            public void run() {
                try { ann.run(); }
                catch(Throwable thr) { stepFailure = thr; }
            }
        }.start();
        timer.reset();
        while (!opened) {
            if (timer.elapsed() > 10000) 
                fail("Annotation window never showed");
        }
        while (!closed) {
            if (timer.elapsed() > ann.getDelayTime() + 1000)
                fail("Annotation window should have closed automatically");
        }
        assertEquals("No exception should be thrown",
                     (Throwable)null, stepFailure);
    }

    public AnnotationTest(String name) { super(name); }

    public static void main(String[] args) {
        TestHelper.runTests(args, AnnotationTest.class);
    }
}

