package abbot.script;

import java.awt.*;
import javax.swing.*;

import junit.extensions.abbot.*;
import junit.framework.*;

public class AssertTest extends ComponentTestFixture {

    /** Verify expressions are always reduced to a canonical boolean form. */
    public void testCanonicalBooleans() {
        String[] args = { };
        Assert step = new Assert(getResolver(), null,
                                 "isSomething", args,
                                 Component.class, "true", false);
        assertEquals("Should expect 'true'",
                     "true", step.getExpectedResult()); 
        assertTrue("Should not be inverted", !step.isInverted());

        step = new Assert(getResolver(), null,
                          "isSomething", args,
                          Component.class, "true", true);
        assertEquals("Should expect 'true'",
                     "true", step.getExpectedResult()); 
        assertTrue("Should be inverted", step.isInverted());

        step = new Assert(getResolver(), null,
                          "isSomething", args,
                          Component.class, "false", false);
        assertEquals("Should expect 'true'",
                     "true", step.getExpectedResult()); 
        assertTrue("Should be inverted", step.isInverted());

        step = new Assert(getResolver(), null,
                          "isSomething", args,
                          Component.class, "false", true);
        assertEquals("Should expect 'true'",
                     "true", step.getExpectedResult()); 
        assertTrue("Should not be inverted", !step.isInverted());
    }

    public void testFixDeprecatedUsage() throws Throwable {
        // A method that is actually on a ComponentTester
        ComponentReference ref =
            new ComponentReference("id", javax.swing.JPopupMenu.class);
        getResolver().addComponentReference(ref);
        String a1 = "<assert component=\"id\" method=\"getMenuLabels\" args=\"id\" value=\"[none]\"/>";
        Step s1 = Step.createStep(getResolver(), a1);
        assertTrue("Not an assert", s1 instanceof Assert);
        assertEquals("Wrong target class", "abbot.tester.JPopupMenuTester",
                     ((Assert)s1).getTargetClassName());
        assertEquals("Component ID should be removed", null,
                     ((Assert)s1).getComponentID());

        // A method that is on a ComponentTester, no component id, no class
        String a2 = "<assert method=\"assertFrameShowing\" args=\"title\"/>";
        Step s2 = Step.createStep(getResolver(), a2);
        assertTrue("Not an assert", s2 instanceof Assert);
        assertEquals("Wrong target class", "abbot.tester.ComponentTester",
                     ((Assert)s2).getTargetClassName());
        // Make sure it will still run with no unexpected errors
        try {
            s1.run();
            s2.run();
        }
        catch(AssertionFailedError afe) {
        }
    }

    public AssertTest(String name) { super(name); }

    public static void main(String[] args) {
        TestHelper.runTests(args, AssertTest.class);
    }
}

