package abbot.script;

import junit.extensions.abbot.TestHelper;

public class CallTest extends junit.framework.TestCase {

    public void testCall() throws Throwable {
        CallTestClass.wasCalled = false;
        Call call = new Call(new Script(), "Test Call",
                             CallTestClass.class.getName(),
                             "someMethod", new String[0]);
        assertTrue("Flag shouldn't be set prior to invocation",
                   !CallTestClass.wasCalled);
        call.run();
        assertTrue("Flag should be set after to invocation",
                   CallTestClass.wasCalled);
    }

    public void testReturnType() throws Throwable {
        CallTestClass.wasCalled = false;
        Call call = new Call(new Script(), "Test Call",
                             CallTestClass.class.getName(),
                             "intReturningMethod", new String[0]);
        assertTrue("Flag shouldn't be set prior to invocation",
                   !CallTestClass.wasCalled);
        call.run();
        assertTrue("Flag should be set after to invocation",
                   CallTestClass.wasCalled);
    }

    public void testEncode() {
        String[][] args = {
            { "1,2", "1\\,2" },
            { ",2", "\\,2" },
            { "2,", "2\\," },
            { "\\,", "\\," },
            { null, "(null)" },
        };
        for (int i=0;i < args.length;i++) {
            assertEquals("String not encoded properly",
                         args[i][1], Call.encode(args[i][0]));
        }
    }

    public CallTest(String name) { super(name); }

    public static void main(String[] args) {
        TestHelper.runTests(args, CallTest.class);
    }
}

class CallTestClass {
    public static boolean wasCalled = false;
    public static void someMethod() {
        wasCalled = true;
    }
    public static int intReturningMethod() {
        wasCalled = true;
        return 0;
    }
}
