package abbot.script;

import javax.swing.*;

import junit.extensions.abbot.*;
import junit.framework.TestCase;
import abbot.Resolver;

/** 
 * Verify the script works as advertised.
 */
public class ComponentReferenceTest extends ComponentTestFixture {

    /** Any newly created reference should match against the component it was
     * created from.
     */
    public void testAddComponent() {
        JTextField tf = new JTextField();
        tf.setColumns(20);
        showFrame(tf);
        ComponentReference ref = new ComponentReference(resolver, tf);
        assertTrue("Newly created component reference does not match",
                   getFinder().componentsMatch(tf, ref,
                                               getFinder().MATCH_EXACT));
    }

    /** Create a reference for a dialog child of a dialog.
        Bug reported by vladgur.
    */
    public void testAddSubDialog() {
        JFrame frame = new JFrame(getName());
        JDialog d1 = new JDialog(frame, "Modal", true);
        d1.getContentPane().add(new JLabel("Modal Dialog"));
        JDialog d2 = new JDialog(d1, "Non-Modal", false);
        d2.getContentPane().add(new JLabel("Non-Modal Dialog"));
        showWindow(d1);
        showWindow(d2);
        ComponentReference ref =
            new ComponentReference(resolver, d2.getContentPane());
        assertNotNull("Can't create subdialog reference");
    }

    public ComponentReferenceTest(String name) {
        super(name);
    }

    private Resolver resolver;
    protected void setUp() {
        resolver = new Script();
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, ComponentReferenceTest.class);
    }
}
