package abbot.tester.awt;

import java.awt.Choice;
import java.awt.event.*;

import junit.extensions.abbot.*;
import junit.framework.*;
import abbot.Platform;

/** Unit test to verify supported operations on the Choice class.<p> */

public class ChoiceTesterTest extends ComponentTestFixture {

    static boolean hasChoiceLockupBug = 
        Platform.isOSX()
        && Platform.getJavaVersionNumber() < Platform.JAVA_1_4;

    public void testSelectIndex() {
        showFrame(choice);
        for (int i=0;i < MAX_ENTRIES;i += MAX_ENTRIES / 10) {
            selectedIndex = -1;
            tester.actionSelectIndex(choice, i);
            assertEquals("Wrong index selected", i, selectedIndex);
            if (hasChoiceLockupBug)
                break;
        }
    }

    public void testSelectItem() {
        showFrame(choice);
        for (int i=0;i < MAX_ENTRIES;i += MAX_ENTRIES / 10) {
            selectedItem = null;
            String item = "item " + i;
            tester.actionSelectItem(choice, item);
            assertEquals("Wrong item selected", item, selectedItem);
            if (hasChoiceLockupBug)
                break;
        }
    }

    private ChoiceTester tester;
    private Choice choice;
    private static final int MAX_ENTRIES = 100;

    /** Create a new test case with the given name. */
    public ChoiceTesterTest(String name) {
        super(name);
    }

    private String selectedItem = null;
    private int selectedIndex = -1;
    protected void setUp() {
        tester = (ChoiceTester)ComponentTester.getTester(Choice.class);
        choice = new Choice();
        for (int i=0;i < MAX_ENTRIES;i++) {
            choice.add("item " + i);
        }
        choice.select(MAX_ENTRIES-1);
        choice.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent ev) {
                abbot.Log.debug("Got " + ev);
                if (ev.getStateChange() == ItemEvent.SELECTED) {
                    selectedIndex = choice.getSelectedIndex();
                    selectedItem = choice.getItem(selectedIndex);
                }
            }
        });
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, ChoiceTesterTest.class);
    }
}
