package abbot.tester.awt;

import junit.extensions.abbot.TestHelper;
import junit.framework.TestCase;

/** Test ArgumentParser functions. */

public class ComparatorTest extends TestCase {

    public void testCompareStrings() {
        String expected1 = "Orange";
        String expected2 = "Black";
        String expected3 = "Ora";
        String expected4 = "O.*"; // will only match something starting with O
        String expected5 = "/^Or....$/"; // will only match Orange
        String arg = "Orange";
        String arg1 = "XOrange";
        assertTrue("String <" + expected1 + "> should match <" + arg + ">",
                   Comparator.stringsMatch(expected1, arg));
        assertTrue("String <" + expected2 + "> should not match <" + arg + ">",
                   !Comparator.stringsMatch(expected2, arg));
        assertTrue("String <" + expected3 + "> should not match <" + arg + ">",
                   !Comparator.stringsMatch(expected3, arg));
        assertTrue("Pattern " + expected4 + " should match " + arg,
                   Comparator.stringsMatch(expected4, arg));
        assertTrue("Pattern " + expected4 + " should not match " + arg1,
                   !Comparator.stringsMatch(expected4, arg1));
        assertTrue("Pattern " + expected5 + " should match " + arg,
                   Comparator.stringsMatch(expected5, arg));
        assertTrue("Pattern " + expected5 + " should not match " + arg1,
                   !Comparator.stringsMatch(expected5, arg1));
    }

    /** Checks array comparisons. */
    public void testArrayEquals() {
        String[] array0 = { };
        String[] array0a = { };
        String[] array1 = { "one" };
        String[] array1a = { "one" };
        String[] array1b = { "six" };
        String[] array2 = { "one", "two" };
        String[] array2a = { "one", "two" };
        String[] array5 = { "one", "two", "three", "four", "five" };
        String[] array5a = { "one", "two", "three", "four", "five" };
        String[] array6 = { "/[0-9]* count/", "Orange", "Black" };
        String[] array6a = { "99 count", "Orange", "Black" };
        String[] array6b = { "99 count", "XOrange", "Black" };

        assertTrue("Array size 0 comparison failed", 
                   Comparator.equals(array0, array0a));
        assertTrue("Array size 0 discomparison failed", 
                   !Comparator.equals(array0, array1));
        assertTrue("Array size 1 comparison failed", 
                   Comparator.equals(array1, array1a));
        assertTrue("Array size 1 discomparison failed", 
                   !Comparator.equals(array1, array1b));
        assertTrue("Array size 1 discomparison failed", 
                   !Comparator.equals(array1, array2));
        assertTrue("Array size 2 comparison failed", 
                   Comparator.equals(array2, array2a));
        assertTrue("Array size 5 comparison failed", 
                   Comparator.equals(array5, array5a));
        assertTrue("Array/object discomparison failed",
                   !Comparator.equals(array5, new Integer(0)));
        assertTrue("Arrays of different types discomparison failed",
                   !Comparator.equals(array5, new Integer[5]));
        assertTrue("String array with regexp pattern should match",
                   Comparator.equals(array6, array6a));
        assertTrue("String array with regexp pattern should not match",
                   !Comparator.equals(array6, array6b));
    }

    /** Create a new test case with the given name. */
    public ComparatorTest(String name) {
        super(name);
    }

    /** Return the default test suite. */
    public static void main(String[] args) {
        TestHelper.runTests(args, ComparatorTest.class);
    }
}
