package abbot.tester.awt;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import junit.extensions.abbot.*;
import junit.extensions.abbot.Timer;
import junit.framework.*;
import abbot.*;

/** Unit test to verify the FrameTester class.<p> */

public class FrameTesterTest extends ComponentTestFixture {

    private FrameTester tester;

    private static final int DX = 300;
    private static final int DY = 240;

    /** Create a new test case with the given name. */
    public FrameTesterTest(String name) {
        super(name);
    }

    protected void setUp() {
        tester = (FrameTester)ComponentTester.getTester(Frame.class);
    }

    public void testMoveBy() {
        Frame frame = showFrame(new JLabel(getName()));
        Point loc = frame.getLocationOnScreen();
        loc.x += DX;
        loc.y += DY;
        tester.actionMoveBy(frame, DX, DY);
        assertEquals("Frame not moved to the desired location",
                     loc, frame.getLocationOnScreen());

        // FIXME +/- coords
        // FIXME scale 5, 10, 50, 100, 500
    }

    // NOTE: may be some timing issues with this test, not sure
    public void testResizeBy() {
        Frame frame = showFrame(new JLabel(getName()));
        Dimension size = frame.getSize();
        size.width += DX;
        size.height += DY;
        tester.actionResizeBy(frame, DX, DY);
        assertEquals("Frame not resized to the desired size",
                     size, frame.getSize());
    }

    private boolean gotClose = false;
    public void testClose() {
        Frame frame = showFrame(new JLabel(getName()));
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                gotClose = true;
            }
        });
        tester.actionClose(frame);
        assertTrue("No WINDOW_CLOSING event generated", gotClose);
    }

    /** Return the default test suite. */
    public static Test suite() {
        return new TestSuite(FrameTesterTest.class);
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, FrameTesterTest.class);
    }
}

