package abbot.tester.awt;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import abbot.*;
import abbot.tester.awt.*;
import junit.extensions.abbot.*;

/** Verify proper operation of tracking input state. */
public class InputStateTest extends ComponentTestFixture {

    private InputState state = null;

    public void testTrackMouse() {
        assertNull("Should be no mouse component w/no components",
                   state.getMouseComponent());
        JButton button = new JButton(getName());
        Frame frame = showFrame(button);
        Point center = new Point(button.getWidth()/2, button.getHeight()/2);
        getRobot().mouseMove(button, center.x, center.y);
        getRobot().waitForIdle();
        assertEquals("Wrong mouse component",
                     Robot.toString(button),
                     Robot.toString(state.getMouseComponent()));
        assertEquals("Wrong relative location",
                     center, state.getMouseLocation());
        Point loc = button.getLocationOnScreen();
        loc.translate(center.x, center.y);
        assertEquals("Wrong screen location",
                     loc, state.getMouseLocationOnScreen());

        getRobot().mouseMove(frame, frame.getWidth() + 1,
                             frame.getHeight() + 1);
        getRobot().waitForIdle();
        assertNull("Should be no mouse component after exit",
                   state.getMouseComponent());
        assertNull("Should be no relative location",
                   state.getMouseLocation());
        assertTrue("Should be a last known screen location",
                   state.getMouseLocationOnScreen() != null);
    }

    public void testTrackDrag() {
        JButton one = new JButton("One");
        JButton two = new JButton("Two");
        JPanel pane = new JPanel();
        pane.add(one);
        pane.add(two);
        Frame frame = showFrame(pane);
        getRobot().drag(one, one.getWidth()/2, one.getHeight()/2,
                        InputEvent.BUTTON1_MASK);
        getRobot().waitForIdle();
        assertEquals("Wrong drag start component",
                     Robot.toString(one),
                     Robot.toString(state.getMouseComponent()));

        getRobot().dragOver(two, two.getWidth()/2, two.getHeight()/2);
        getRobot().waitForIdle();
        assertEquals("Containing component should be tracked during drag",
                     Robot.toString(two),
                     Robot.toString(state.getMouseComponent()));

        getRobot().dragOver(frame, frame.getWidth() + 1,
                            frame.getHeight() + 1);
        getRobot().waitForIdle();
        assertNull("Drag outside should leave no mouse component",
                   state.getMouseComponent());
    }

    protected void setUp() {
        state = new InputState();
    }

    protected void tearDown() {
        state.dispose();
        state = null;
    }

    public InputStateTest(String name) { super(name); }
    public static void main(String[] args) {
        TestHelper.runTests(args, InputStateTest.class);
    }
}
