package abbot.tester.awt;

import java.awt.Rectangle;

import javax.swing.*;

import junit.extensions.abbot.*;
import abbot.Log;

/** Unit test to verify the base ComponentTester class.<p> */

public class JComponentTesterTest extends ComponentTestFixture {

    private JComponentTester tester;

    /** Create a new test case with the given name. */
    public JComponentTesterTest(String name) {
        super(name);
    }

    protected void setUp() {
        tester = (JComponentTester)ComponentTester.getTester(JComponent.class);
    }

    private void scrollAndAssertVisible(JList list, int row) {
        Rectangle rect = list.getCellBounds(row, row);
        tester.actionScrollToVisible(list, rect.x, rect.y,
                                     rect.width, rect.height);
        tester.actionWaitForIdle();
        Rectangle visible = list.getVisibleRect();
        Log.debug("Visible rect: " + visible);
        assertTrue("Horizontal scroll to element " + row + " failed ("
                   + visible.x + " <= " + rect.x + " < " + (visible.x +
                                                            visible.width)
                   + ")",
                   visible.x <= rect.x
                   && rect.x < visible.x + visible.width);
        assertTrue("Vertical scroll to element " + row + " failed ("
                   + visible.y + " <= " + rect.y + " < " + (visible.y +
                                                            visible.height)
                   + ")",
                   visible.y <= rect.y
                   && rect.y < visible.y + visible.height);
    }

    /** Test scrolling for JComponents within a scroll pane. */
    // FIXME really needs to scroll horizontally as well
    public void testScroll() {
        String[] data = {
            "zero", "one", "two", "three", "four", 
            "five", "six", "seven", "eight", "nine",
            "zero", "one", "two", "three", "four", 
            "five", "six", "seven", "eight", "nine",
            "zero", "one", "two", "three", "four", 
            "five", "six", "seven", "eight", "nine",
            "zero", "one", "two", "three", "four", 
            "five", "six", "seven", "eight", "nine",
            "zero", "one", "two", "three", "four", 
            "five", "six", "seven", "eight", "nine",
        };
        JList list = new JList(data);
        JScrollPane scroll = new JScrollPane(list);
        showFrame(scroll);

        scrollAndAssertVisible(list, 0);
        scrollAndAssertVisible(list, data.length / 4);
        scrollAndAssertVisible(list, data.length / 2);
        scrollAndAssertVisible(list, data.length * 3 / 4);
        scrollAndAssertVisible(list, data.length / 2);
        scrollAndAssertVisible(list, data.length / 4);
        scrollAndAssertVisible(list, 0);
    }

    public void testActionMap() {
        // Use something for which we know the action/mappings
        JTextField tf = new JTextField("Some text");
        showFrame(tf);
        tester.actionFocus(tf);
        tester.actionActionMap(tf, "select-all");
        assertEquals("Selection should start at beginning of text",
                     0, tf.getSelectionStart());
        assertEquals("Selection should end at end of text",
                     tf.getText().length(), tf.getSelectionEnd());
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JComponentTesterTest.class);
    }
}
