package abbot.tester.awt;

import javax.swing.*;
import javax.swing.text.JTextComponent;

import junit.extensions.abbot.*;

/** Unit test to verify the JTextComponentTester class.<p> */

public class JTextComponentTesterTest extends ComponentTestFixture {

    private JTextComponentTester tester = (JTextComponentTester)
        ComponentTester.getTester(JTextComponent.class);
    private JTextComponent tc;
    private JScrollPane scrollPane;

    /** Wholesale text replacement by typing. */
    public void testSetText() {
        showFrame(tc);
        String text = "short";
        tester.actionEnterText(tc, text);
        assertEquals("Wrong short text typed,", text, tc.getText());
        text = "longer";
        tester.actionEnterText(tc, text);
        assertEquals("Wrong replacement text,",
                     text, tc.getText());
        text = "Some longer text that will surely exceed the field width";
        tester.actionEnterText(tc, text);
        assertEquals("Wrong long replacement text,",
                     text, tc.getText());
        text = "shorter";
        tester.actionEnterText(tc, text);
        assertEquals("Wrong shorter replacement text,",
                     text, tc.getText());

    }

    public void testClick() {
        showFrame(tc);
        String text = "Some somewhat long text which exceeds the component size";
        tc.setText(text);
        tester.waitForIdle();
        tester.actionClick(tc, 0);
        assertEquals("Wrong location", 0, tc.getCaretPosition());
        tester.actionClick(tc, text.length() / 2);
        assertEquals("Wrong location",
                     text.length() / 2, tc.getCaretPosition());
        tester.actionClick(tc, text.length());
        assertEquals("Wrong location", text.length(), tc.getCaretPosition());
    }

    /** Verify we can select arbitrary bits of text. */
    public void testSelect() {
        showFrame(tc);
        String text = "short";
        tc.setText(text);
        tester.waitForIdle();
        tester.actionSelect(tc, 0, text.length());
        assertEquals("Wrong selection start",
                     0, tc.getSelectionStart());
        assertEquals("Wrong selection end",
                     text.length(), tc.getSelectionEnd());

        tester.actionSelect(tc, 1, text.length()-1);
        assertEquals("Wrong selection start",
                     1, tc.getSelectionStart());
        assertEquals("Wrong selection end",
                     text.length()-1, tc.getSelectionEnd());
    }
    
    public void testLongSelection() {
        showFrame(tc);
        // Try a long selection which exceeds the visible area
        String text = "The quick brown fox jumped over the lazy dog";
        tc.setText(text);
        tester.waitForIdle();
        tester.actionSelect(tc, 0, text.length());
        assertEquals("Wrong selection start",
                     0, tc.getSelectionStart());
        assertEquals("Wrong selection end",
                     text.length(), tc.getSelectionEnd());
        tester.actionSelect(tc, 1, text.length()-1);
        assertEquals("Wrong selection start",
                     1, tc.getSelectionStart());
        assertEquals("Wrong selection end",
                     text.length()-1, tc.getSelectionEnd());
    }

    /** Create a new test case with the given name. */
    public JTextComponentTesterTest(String name) {
        super(name);
    }

    protected void setUp() {
        JTextField tf = new JTextField();
        tf.setColumns(10);
        tc = tf;
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JTextComponentTesterTest.class);
    }
}

