package abbot.tester.awt;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import junit.extensions.abbot.*;

/** Unit test to verify the JTreeTester class.<p> */

public class JTreeTesterTest extends ComponentTestFixture {

    private JTreeTester tester = (JTreeTester)
        ComponentTester.getTester(JTree.class);
    private JTree tree;
    private JScrollPane scrollPane;

    public void testConvertPath() {
        TreePath stringPath = new TreePath(new String[] {
            "root", "parent 2", "child 2"
        });
        TreePath path = tester.convertPath(tree.getModel(), stringPath);
        assertNotNull("Couldn't convert path", path);
        assertEquals("Wrong path",
                     "[root, parent 2, child 2]", path.toString());
    }

    public void testMakeVisible() {
        showFrame(scrollPane);
        TreePath stringPath = new TreePath(new String[] {
            "root", "parent 4", "child 3"
        });
        TreePath path = tester.convertPath(tree.getModel(), stringPath);
        assertNotNull("Couldn't convert path", path);
        tester.actionMakeVisible(tree, path);
        assertTrue("Path not visible", tree.isVisible(path));

        stringPath = new TreePath(new String[] {
            "root", "parent 4", "child 3", "leaf 1"
        });
        path = tester.convertPath(tree.getModel(), stringPath);
        assertNotNull("Couldn't convert path", path);
        tester.actionMakeVisible(tree, path);
        assertTrue("Path not visible", tree.isVisible(path));
    }

    public void testSelectPath() {
        showFrame(scrollPane);
        TreePath stringPath = new TreePath(new String[] {
            "root", "parent 2", "child 2"
        });
        TreePath path = tester.convertPath(tree.getModel(), stringPath);
        assertNotNull("Couldn't convert path", path);

        tester.actionSelectPath(tree, stringPath);
        TreePath[] paths = tree.getSelectionPaths();
        assertTrue("Paths not visible", tree.isVisible(path));
        assertTrue("No paths selected", paths != null);
        assertEquals("Too many selected", 1, paths.length);
        assertEquals("Wrong path selected", stringPath.toString(), paths[0].toString());
    }

    public void testSelectRow() {
        int row = 1;
        showFrame(scrollPane);
        tester.actionSelectRow(tree, row);
        TreePath[] paths = tree.getSelectionPaths();
        assertTrue("No paths selected", paths != null);
        assertEquals("Wrong row count selected", 1, paths.length);
        assertEquals("Wrong row selected", tree.getPathForRow(row), paths[0]);
    }

    public void testToggleRow() {
        int row = 2;
        showFrame(scrollPane);
        tester.actionToggleRow(tree, row);
        assertTrue("Node should have expanded", tree.isExpanded(row));
        // Avoid the next action being interpreted as the same click
        tester.delay(500);
        tester.actionToggleRow(tree, row);
        assertTrue("Node should not still be expanded", !tree.isExpanded(row));
    }

    /** Create a new test case with the given name. */
    public JTreeTesterTest(String name) {
        super(name);
    }

    protected void setUp() {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");
        DefaultMutableTreeNode parent, child, leaf;
        int COUNT = 5;
        for (int i=0;i < COUNT;i++) {
            parent = new DefaultMutableTreeNode("parent " + i);
            root.add(parent);
            for (int j=0;j < COUNT;j++) {
                child = new DefaultMutableTreeNode("child " + j);
                parent.add(child);
                for (int k=0;k < COUNT;k++) {
                    leaf = new DefaultMutableTreeNode("leaf " + k, false);
                    child.add(leaf);
                }
            }
        }
        tree = new JTree(root);
        scrollPane = new JScrollPane(tree);
    }

    public static void main(String[] args) {
        RepeatHelper.runTests(args, JTreeTesterTest.class);
    }
}

