package abbot.tester.awt;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;

import junit.extensions.abbot.*;
import junit.extensions.abbot.Timer;

public class RobotAWTModeTest extends ComponentTestFixture {

    private Robot robot;

    public void testBorderIntersection() {
        Rectangle rect = new Rectangle(0, 0, 100, 100);
        Point center = new Point(rect.width/2, rect.height/2);

        Point outside = new Point(-1, rect.height/2);
        assertEquals("Left intersection incorrect",
                     new Point(0, rect.height/2),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(-1, -1);
        assertEquals("Upper left intersection incorrect",
                     new Point(0, 0),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(rect.width/2, -1);
        assertEquals("Top intersection incorrect",
                     new Point(rect.width/2, 0),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(rect.width+1, -1);
        assertEquals("Upper Right intersection incorrect",
                     new Point(rect.width, 0),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(rect.width+1, rect.height/2);
        assertEquals("Right intersection incorrect",
                     new Point(rect.width, rect.height/2),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(rect.width+1, rect.height+1);
        assertEquals("Lower Right intersection incorrect",
                     new Point(rect.width, rect.height),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(rect.width/2, rect.height+1);
        assertEquals("Bottom intersection incorrect",
                     new Point(rect.width/2, rect.height),
                     robot.getBorderIntersection(rect, center, outside));
        outside = new Point(-1, rect.height+1);
        assertEquals("Lower left intersection incorrect",
                     new Point(0, rect.height),
                     robot.getBorderIntersection(rect, center, outside));
    }

    private class EnterExitListener implements AWTEventListener {
        public boolean gotExit = false;
        public Component exited = null;
        public boolean gotEnter = false;
        public Component entered = null;
        public void eventDispatched(AWTEvent event) {
            if (event.getID() == MouseEvent.MOUSE_ENTERED) {
                gotEnter = true;
                entered = (Component)event.getSource();
            }
            else if (event.getID() == MouseEvent.MOUSE_EXITED) {
                gotExit = true;
                exited = (Component)event.getSource();
            }
        }
    }

    public void testEnterExitGeneration() {
        robot.setEventMode(Robot.EM_AWT);
        EnterExitListener eel = new EnterExitListener();
        try {
            JPanel panel = new JPanel();
            JLabel label1 = new JLabel("Source");
            JLabel label2 = new JLabel("Destination");
            panel.add(label1);
            panel.add(label2);
            showFrame(panel, new Dimension(200, 200));
            // Ensure we have a state to start with
            robot.mouseMove(label1);
            robot.waitForIdle();
            // Add the listener *after* the mouse is within the first component
            Toolkit.getDefaultToolkit().
                addAWTEventListener(eel, MouseEvent.MOUSE_EVENT_MASK);
            robot.mouseMove(label2);
            robot.waitForIdle();
            assertTrue("Got no exit event", eel.gotExit);
            assertEquals("Wrong exit component", label1, eel.exited);
            assertTrue("Got no enter event", eel.gotEnter);
            assertEquals("Wrong enter component", label2, eel.entered);
        }
        finally {
            robot.setEventMode(Robot.EM_ROBOT);
            Toolkit.getDefaultToolkit().removeAWTEventListener(eel);
        }
    }

    public RobotAWTModeTest(String name) { super(name); }

    private int lastMode;
    /** All these tests run in AWT mode. */
    protected void setUp() {
        robot = getRobot();
        lastMode = robot.getEventMode();
        robot.setEventMode(Robot.EM_AWT);
    }

    /** Restore the robot event generation mode. */
    protected void tearDown() {
        robot.setEventMode(lastMode);
    }

    /** Provide for repetitive testing on individual tests. */
    public static void main(String[] args) {
        RepeatHelper.runTests(args, RobotAWTModeTest.class);
    }
}
