package abbot.tester.awt;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;

import junit.extensions.abbot.*;
import junit.extensions.abbot.Timer;
import abbot.*;

/** Unit test to verify Robot operation.<p>

    Also add tests here for all known robot bugs (jitter, frame motion, etc.).
 */

public class RobotTest extends ComponentTestFixture {

    /** Ensure image capture gets the right image. */
    public void testImageCapture() throws Throwable {
        final int X = 100;
        final int Y = 100;
        File gif = new File("test/abbot/tester/image.gif");
        ImageIcon icon = new ImageIcon(gif.toURL());
        JLabel label1 = new JLabel(icon);
        JLabel label2 = new JLabel(icon);
        final JFrame frame = new JFrame(getName());
        frame.setLocation(X, Y);
        JPanel pane = (JPanel)frame.getContentPane();
        pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
        pane.add(label1);
        pane.add(label2);
        frame.pack();
        frame.setResizable(false);
        frame.setLocation(X, Y);
        showWindow(frame);
        robot.focus(frame);
        robot.waitForIdle();
        
        File jpg = new File("test/abbot/tester/image.jpg");
        ImageComparator ic = new ImageComparator();
        assertTrue("Captured wrong image (1)",
                   ic.compare(robot.capture(label1), jpg) == 0);
        assertTrue("Captured wrong image (2)",
                   ic.compare(robot.capture(label2), jpg) == 0);
    }

    private boolean f1 = false;
    private boolean f2 = false;
    public void testAWTMenuSelection() {
        MenuBar mb = new MenuBar();
        Menu menu = new Menu("File");
        menu.add(new MenuItem("One"));
        MenuItem mi = new MenuItem("Two");
        mi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) { f1 = true; }
        });
        menu.add(mi);
        Menu sub = new Menu("Submenu");
        sub.add(new MenuItem("Sub one"));
        MenuItem mi2 = new MenuItem("Sub two");
        mi2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) { f2 = true; }
        });
        sub.add(mi2);
        menu.add(sub);
        mb.add(menu);
        JFrame frame = new JFrame(getName());
        frame.setMenuBar(mb);
        showWindow(frame);
	
	    robot.selectAWTMenuItem(mi);
        robot.waitForIdle();
        assertTrue("AWT Menu item in menubar menu not hit", f1);
        robot.selectAWTMenuItem(mi2);
        robot.waitForIdle();
        assertTrue("AWT Menu item in menubar submenu not hit", f2);

        f1 = f2 = false;
        robot.selectAWTMenuItemByLabel(frame, mi.getLabel());
        robot.waitForIdle();
        assertTrue("AWT Menu item in menubar menu not hit", f1);
        robot.selectAWTMenuItemByLabel(frame, mi2.getLabel());
        robot.waitForIdle();
        assertTrue("AWT Menu item in menubar submenu not hit", f2);
    }


    private class MenuListener implements ActionListener {
        public boolean gotAction = false;
        public void actionPerformed(ActionEvent ev) {
            gotAction = true;
        }
    }
    public void testMenuSelection() {
        JMenuBar mb = new JMenuBar();
        JMenu menu = new JMenu("File");
        menu.add(new JMenuItem("One"));
        JMenuItem mi1 = new JMenuItem("Two");
        MenuListener ml1 = new MenuListener();
        mi1.addActionListener(ml1);
        menu.add(mi1);
        JMenu sub = new JMenu("Submenu");
        sub.add(new JMenuItem("Sub one"));
        JMenuItem mi2 = new JMenuItem("Sub two");
        MenuListener ml2 = new MenuListener();
        mi2.addActionListener(ml2);
        sub.add(mi2);
        menu.add(sub);
        mb.add(menu);
        JFrame frame = new JFrame(getName());
        frame.getContentPane().add(new JLabel(getName()));
        frame.setJMenuBar(mb);
        showWindow(frame);
	
        robot.selectMenuItem(mi1);
        assertTrue("Menu item in menubar menu not hit", ml1.gotAction);
        robot.selectMenuItem(mi2);
        assertTrue("Menu item in menubar submenu not hit", ml2.gotAction);
    }

    public void testMenuSelectionWithParentShowing() {
        JMenuBar mb = new JMenuBar();
        JMenu menu = new JMenu("File");
        menu.add(new JMenuItem("One"));
        JMenuItem mi1 = new JMenuItem("Two");
        MenuListener ml1 = new MenuListener();
        mi1.addActionListener(ml1);
        menu.add(mi1);
        mb.add(menu);

        JFrame frame = new JFrame(getName());
        frame.getContentPane().add(new JLabel(getName()));
        frame.setJMenuBar(mb);
        showWindow(frame);

        robot.click(menu);
        robot.waitForIdle();
        robot.selectMenuItem(mi1);
        assertTrue("Menu it main menu not hit when parent already open",
                   ml1.gotAction);
    }

    private class MouseWatcher extends MouseAdapter {
        public boolean gotPress = false;
        public boolean gotRelease = false;
        public boolean gotClick = false;
        public int clickCount = 0;
        public boolean popupTrigger = false;
        public Component source = null;
        public int modifiers = 0;
        public Point where = null;
        public void mousePressed(MouseEvent me) {
            gotPress = true;
            source = me.getComponent();
            popupTrigger = me.isPopupTrigger();
            modifiers = me.getModifiers();
            where = me.getPoint();
        }
        public void mouseReleased(MouseEvent me) {
            gotRelease = true;
            popupTrigger = popupTrigger || me.isPopupTrigger();
        }
        public void mouseClicked(MouseEvent me) {
            gotClick = true;
            clickCount = me.getClickCount();
        }
    }

    /** Verify proper click operation.  You may need to set
        Robot.mouseReleaseDelay if this fails intermittently. */
    public void testClick() {
        MouseWatcher mw = new MouseWatcher();
        JFrame frame = new JFrame(getName());
        frame.addMouseListener(mw);
        showWindow(frame, new Dimension(200, 200), true);
        robot.click(frame);
        robot.waitForIdle();
        assertTrue("Never received press", mw.gotPress);
        assertEquals("Wrong event source", frame, mw.source);
        assertTrue("Never received release", mw.gotRelease);
        assertTrue("Never received click", mw.gotClick);
        assertEquals("No modifiers expected", 
                     robot.getMouseModifiers(MouseEvent.BUTTON1_MASK), 
                     robot.getMouseModifiers(mw.modifiers));
        assertTrue("Unexpected popup trigger", !mw.popupTrigger);
    }

    /** Double clicks. */
    public void testMultipleClicks() {
        MouseWatcher mw = new MouseWatcher();
        JFrame frame = new JFrame(getName());
        showWindow(frame, new Dimension(200, 200));
        JPanel pane = (JPanel)frame.getContentPane();
        pane.addMouseListener(mw);
        robot.click(pane, 
                    pane.getWidth()/2,
                    pane.getHeight()/2,
                    InputEvent.BUTTON1_MASK, 2);
        Timer timer = new Timer();
        while (mw.clickCount < 2) {
            if (timer.elapsed() > 5000)
                fail("Never received a double click");
        }
    }

    public void testClickAt() {
        MouseWatcher mw = new MouseWatcher();
        JFrame frame = new JFrame(getName());
        frame.addMouseListener(mw);
        showWindow(frame, new Dimension(200, 200));
        // Make sure we don't click in the title or border...
        Insets insets = frame.getInsets();
        Point at = new Point(insets.left,insets.top);
        robot.click(frame, at.x, at.y);
        robot.waitForIdle();
        assertTrue("Never received press", mw.gotPress);
        assertEquals("Wrong event source", frame, mw.source);
        assertTrue("Never received release", mw.gotRelease);
        assertTrue("Never received click", mw.gotClick);
        assertEquals("Wrong mouse coordinates", at, mw.where);
        assertEquals("No modifiers expected", 
                     robot.getMouseModifiers(MouseEvent.BUTTON1_MASK), 
                     robot.getMouseModifiers(mw.modifiers));
        assertTrue("Unexpected popup trigger", !mw.popupTrigger);
    }

    public void testClickPopup() {
        MouseWatcher mw = new MouseWatcher();
        JFrame frame = new JFrame(getName());
        frame.addMouseListener(mw);
        showWindow(frame, new Dimension(200, 200));
        robot.click(frame, robot.POPUP_MASK);
        robot.waitForIdle();
        assertTrue("Never received press", mw.gotPress);
        assertEquals("Wrong event source", frame, mw.source);
        assertTrue("Never received release", mw.gotRelease);
        assertTrue("Never received click", mw.gotClick);
        assertEquals("Wrong modifiers", 
                     robot.getMouseModifiers(robot.POPUP_MASK), 
                     robot.getMouseModifiers(mw.modifiers));
        assertTrue("Should be popup trigger", mw.popupTrigger);
    }

    public void testClickTertiary() {
        MouseWatcher mw = new MouseWatcher();
        JFrame frame = new JFrame(getName());
        frame.addMouseListener(mw);
        showWindow(frame, new Dimension(200, 200));
        int mask = robot.TERTIARY_MASK;
        robot.click(frame, mask);
        robot.waitForIdle();
        // w32 uses CTRL as a modifier prior to 1.4, I guess in case you don't
        // have a 3-button mouse.
        if (Platform.isWindows()
            && Platform.getJavaVersionNumber() < Platform.JAVA_1_4) {
            mask |= InputEvent.CTRL_MASK;
        }

        assertTrue("Never received press", mw.gotPress);
        assertEquals("Wrong event source", frame, mw.source);
        assertTrue("Never received release", mw.gotRelease);
        assertTrue("Never received click", mw.gotClick);
        assertEquals("Wrong modifiers", 
                     robot.getMouseModifiers(mask),
                     robot.getMouseModifiers(mw.modifiers));
        assertTrue("Unexpected popup trigger", !mw.popupTrigger);
    }

    public void testFocusSingleFrame() {
        JTextField tf = new JTextField("Default focus goes here");
        JTextField tf2 = new JTextField("Requested focus goes here");
        JPanel pane = new JPanel();
        pane.add(tf);
        pane.add(tf2);
        showFrame(pane);
        robot.focus(tf2);
        robot.waitForIdle();
        assertTrue("Default TextField should not have focus", !tf.hasFocus());
        assertTrue("Next TextField should have focus", tf2.hasFocus());
        robot.focus(tf);
        robot.waitForIdle();
        assertTrue("First TextField should have focus", tf.hasFocus());
    }

    public void testFocusMultipleFrames() {
        JTextField tf = new JTextField("Default focus goes here");
        JTextField tf2 = new JTextField("Requested focus goes here");
        JTextField tf3 = new JTextField("Next it goes here");
        JPanel pane = new JPanel();
        pane.add(tf);
        pane.add(tf2);
        showFrame(pane);
        JFrame frame = new JFrame(getName() + " 2");
        frame.getContentPane().add(tf3);
        showWindow(frame);
        robot.focus(tf2);
        Timer timer = new Timer();
        while (!tf2.hasFocus()) {
            if (timer.elapsed() > 5000) {            
                fail("Next Textfield never got focus");
            }
        }
        robot.focus(tf3);
        timer.reset();
        while (!tf3.hasFocus()) {
            if (timer.elapsed() > 5000) {            
                fail("Textfield on second frame never got focus");
            }
        }
    }

    private class KeyWatcher extends KeyAdapter {
        public boolean gotPress = false;
        public boolean gotRelease = false;
        public boolean gotTyped = false;
        public int keyCode = KeyEvent.VK_UNDEFINED;
        public int modifiers = 0;
        public void keyPressed(KeyEvent ke) {
            gotPress = true;
            keyCode = ke.getKeyCode();
            modifiers = ke.getModifiers();
        }
        public void keyReleased(KeyEvent ke) {
            gotRelease = true;
        }
        public void keyTyped(KeyEvent ke) {
            gotTyped = true;
        }
    }

    public void testPlainKey() {
        KeyWatcher kw = new KeyWatcher();
        JTextField tf = new JTextField();
        tf.setColumns(10);
        tf.addKeyListener(kw);
        JPanel pane = new JPanel();
        pane.add(tf);
        showFrame(pane);
        robot.focus(tf);
        int code = KeyEvent.VK_A;
        robot.key(code);
        robot.waitForIdle();
        assertTrue("Never received key press", kw.gotPress);
        assertTrue("Never received release", kw.gotRelease);
        assertTrue("Never received type", kw.gotTyped);
        assertEquals("Wrong key code", code, kw.keyCode);
        assertEquals("Wrong modifiers", 0, kw.modifiers);
    }

    public void testKeyString() {
        JPanel pane = new JPanel();
        JTextField tf1 = new JTextField();
        JTextField tf2 = new JTextField();
        tf1.setColumns(10);
        tf2.setColumns(10);
        pane.add(tf1);
        pane.add(tf2);
        Frame frame = showFrame(pane);
        robot.activate(frame);
        robot.waitForIdle();
        robot.focus(tf1);
        String keymap = "The quick brown fox jumped over the lazy dog";
        robot.keyString(keymap);
        robot.waitForIdle();
        assertEquals("Wrong text typed", keymap, tf1.getText());
    

		String string = "Un \u00e9l\u00e9ment gr\u00e2ce \u00e0 l'index";
		robot.focus(tf2);
		robot.waitForIdle();
		robot.keyString(string);
		robot.waitForIdle();
		assertEquals("Wrong non-keymap text typed", string, tf2.getText());
    }

    public void testFindFocusOwner() {
        JPanel pane = new JPanel();
        JTextField tf1 = new JTextField("tf 1");
        tf1.setName("tf 1");
        JTextField tf2 = new JTextField("tf 2");
        tf2.setName("tf 2");
        pane.add(tf1);
        pane.add(tf2);
        showFrame(pane);
        robot.focus(tf2);
        robot.waitForIdle();
        assertEquals("Wrong focus detected",
                     Robot.toString(tf2),
                     Robot.toString(robot.findFocusOwner()));
    }

    public void testIconify() throws Throwable {
        Frame frame = showFrame(new JLabel(getName()));
        robot.iconify(frame);
        // Give the WM time to put the window away
        Timer timer = new Timer();
        while (frame.getState() != Frame.ICONIFIED) {
            if (timer.elapsed() > 2000)
                fail("Frame not iconified");
            robot.sleep();
        }
        robot.deiconify(frame);
        timer.reset();
        while (frame.getState() != Frame.NORMAL) {
            if (timer.elapsed() > 2000)
                fail("Frame not restored");
            robot.sleep();
        }
    }

    public void testMaximize() {
        Frame frame = showFrame(new JLabel(getName()));
        Dimension size = frame.getSize();
        robot.maximize(frame);
        Timer timer = new Timer();
        while (frame.getSize().equals(size)) {
            if (timer.elapsed() > 2000)
                fail("Frame not maximized");
            robot.sleep();
        }
        robot.normalize(frame);
        // Don't bother testing this (it won't work on 1.3.1)
    }

    /** Create a new test case with the given name. */
    public RobotTest(String name) {
        super(name);
    }

    private Robot robot;
    protected void setUp() {
        robot = getRobot();
    }

    /** Provide for repetitive testing on individual tests. */
    public static void main(String[] args) {
        RepeatHelper.runTests(args, RobotTest.class);
    }
}
