package abbot.util;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import abbot.Log;
import abbot.tester.awt.*;
import abbot.tester.awt.Robot;
import junit.extensions.abbot.*;
import junit.framework.*;

public class AWTTest extends ComponentTestFixture {

    public void testReenable() {
        JButton b1 = new JButton("Button 1 (disabled)");
        b1.setEnabled(false);
        JButton b2 = new JButton("Button 2 (enabled)");
        JPanel pane = new JPanel();
        pane.setBorder(new TitledBorder(getName() + "enabled"));
        pane.add(b1);
        pane.add(b2);
        JPanel pane2 = new JPanel();
        pane2.setBorder(new TitledBorder(getName() + "disabled"));
        pane2.setEnabled(false);

        JPanel content = new JPanel();
        content.setLayout(new BorderLayout());
        content.add(pane, BorderLayout.NORTH);
        content.add(pane2);

        Frame frame = showFrame(content);

        java.util.List list = AWT.disableHierarchy(frame);
        assertTrue("1st pane should be disabled", !pane.isEnabled());
        assertTrue("2nd pane should be disabled", !pane2.isEnabled());
        assertTrue("1st button should be disabled", !b1.isEnabled());
        assertTrue("2nd button should be disabled", !b2.isEnabled());
        assertTrue("content should be disabled", !content.isEnabled());
        assertTrue("frame should be disenabled", !frame.isEnabled());
        
        AWT.reenableHierarchy(list);
        getRobot().waitForIdle();
        assertTrue("1st pane should be re-enabled", pane.isEnabled());
        assertTrue("2nd pane should be disabled", !pane2.isEnabled());
        assertTrue("1st button should be disabled", !b1.isEnabled());
        assertTrue("2nd button should be re-enabled", b2.isEnabled());
        assertTrue("content should be re-enabled", content.isEnabled());
        assertTrue("frame should be re-enabled", frame.isEnabled());
    }

    public void testLightweightTransientPopups() {
        Frame frame = showFrame(new JLabel(getName()),
                                new Dimension(200, 200));
        JPanel pane = (JPanel)((JFrame)frame).getContentPane();
        JPopupMenu popup = new JPopupMenu("menu");
        popup.add(new JMenuItem("item 1"));
        popup.add(new JMenuItem("item 2"));
        showPopup(popup, pane, 1, 1);
        Component container = popup.getParent();
        Log.debug("Popup parent is " + Robot.toString(container)
                  + " (" + container.getClass() + ")");
        while (!AWT.isTransientPopup(container)) {
            if (container.getParent() == null)
                throw new AssertionFailedError("No transient popup ancestor to "
                                               + Robot.toString(popup)
                                               + " (ancestor " 
                                               + Robot.toString(container)
                                               + ")");
            container = container.getParent();
            Log.debug("Next ancestor is " + Robot.toString(container));
        }
        assertTrue("No transient popup ancestor detected for "
                   + Robot.toString(container),
                   AWT.isTransientPopup(container));
        // Not actually guaranteed to get a lightweight popup
    }
        
    public void testHeavyweightTransientPopups() {
        Frame frame = showFrame(new JLabel(getName()),
                                new Dimension(200, 200));
        JPanel pane = (JPanel)((JFrame)frame).getContentPane();
        JPopupMenu popup = new JPopupMenu("menu");
        popup.add(new JMenuItem("item 1"));
        popup.add(new JMenuItem("item 2"));
        popup.add(new JMenuItem("item 3"));
        popup.add(new JMenuItem("item 4"));
        popup.add(new JMenuItem("item 5"));
        popup.add(new JMenuItem("item 6"));
        showPopup(popup, pane, 1, pane.getHeight()-5);

        Component container = popup.getParent();
        Log.debug("Popup parent is " + Robot.toString(container)
                  + " (" + container.getClass() + ")");
        while (!AWT.isTransientPopup(container)) {
            if (container.getParent() == null)
                throw new AssertionFailedError("No transient popup ancestor to "
                                               + Robot.toString(popup)
                                               + " (ancestor " 
                                               + Robot.toString(container)
                                               + ")");
            container = container.getParent();
            Log.debug("Next ancestor is " + Robot.toString(container));
        }
        assertTrue("No transient popup ancestor detected for "
                   + Robot.toString(container),
                   AWT.isTransientPopup(container));
        // might be heavyweight, might not
        assertTrue(Robot.toString(container) + " should be a Window",
                   container instanceof Window);
    }

    public AWTTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, AWTTest.class);
    }
}
