package abbot.util;

import java.awt.EventQueue;
import junit.extensions.abbot.TestHelper;
import junit.extensions.abbot.Timer;
import junit.framework.TestCase;
import junit.framework.AssertionFailedError;

public class AbstractEventExceptionHandlerTest extends TestCase {

    private boolean flag = false;
    private Runnable empty = new Runnable() { public void run(){ } };

    /** Ensure we can install an instance and catch an exception. */
    public void testCatchException() throws Throwable {
        // ensure the standard handler is always installed first
        AbstractEventExceptionHandler.install(Catcher.class);
        Catcher.throwable = null;
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                throw new RuntimeException("Test exception for " + getName());
            }
        });
        EventQueue.invokeAndWait(empty);
        assertNotNull("No exception caught", Catcher.throwable);
    }

    /** Ensure subsequently set handlers will get called. */
    public void testForwardException() throws Throwable {
        // ensure the standard handler is always installed first
        AbstractEventExceptionHandler.install(Catcher.class);
        Catcher.throwable = null;
        String oldHandler =
            System.getProperty(AbstractEventExceptionHandler.PROP_NAME);
        SampleCatcher.throwable = null;
        System.setProperty(AbstractEventExceptionHandler.PROP_NAME,
                           SampleCatcher.class.getName());
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                throw new RuntimeException("Test exception for " + getName());
            }
        });
        EventQueue.invokeAndWait(empty);
        try {
            // Only one handler can be installed at a time, so if there was
            // already a handler installed, this handler won't see the
            // exception. 
            //assertNotNull("No exception caught", Catcher.throwable);
            assertNotNull("Exception not forwarded", SampleCatcher.throwable);
        }
        finally {
            System.setProperty(AbstractEventExceptionHandler.PROP_NAME,
                               oldHandler);
        }
    }
    
    public AbstractEventExceptionHandlerTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, AbstractEventExceptionHandlerTest.class);
    }

    public static class Catcher extends AbstractEventExceptionHandler {
        public static Throwable throwable = null;
        public void exceptionCaught(Throwable thr) {
            throwable = thr;
        }
    }

    public static class SampleCatcher {
        public static Throwable throwable = null;
        public void handle(Throwable thr) {
            throwable = thr;
        }
    }
}
