package abbot.util;

import java.io.File;

import junit.extensions.abbot.TestHelper;
import junit.framework.TestCase;
import abbot.Platform;

public class PathClassLoaderTest extends TestCase {

    public void testConvertPathToFiles() {
        if (Platform.isWindows()) {
            String path = "c:\\;c:\\mydir;.";
            File[] files = PathClassLoader.convertPathToFiles(path, ":;", true);
            String[] expected = {
                "c:\\", "c:\\mydir", "."
            };
            assertEquals("Wrong number of files",
                         expected.length, files.length);
            for (int i=0;i < files.length;i++) {
                assertEquals("Wrong path", expected[i], files[i].getPath());
            }
        }
        else {
            String path = "/:/tmp:/tmp/mydir:.";
            File[] files = PathClassLoader.convertPathToFiles(path, ":;", true);
            String[] expected = {
                "/", "/tmp", "/tmp/mydir", "."
            };
            assertEquals("Wrong number of files",
                         expected.length, files.length);
            for (int i=0;i < files.length;i++) {
                assertEquals("Wrong path", expected[i], files[i].getPath());
            }
        }
    }

    /** We assume the existence of "lib/example.jar" and a resource
     * example/logo32.gif within it.
     */ 
    public void testLoadResource() throws Throwable {
        String pathName = "lib/example.jar";
        String rsrc = "/example/logo32.gif";
        ClassLoader cl = new PathClassLoader(pathName,
                                             getClass().getClassLoader());
        try {
            cl.getResourceAsStream(rsrc);
        }
        catch(Exception exc) {
            fail("Resource not found: " + exc.getMessage());
        }
    }

    /** We assume the existence of "lib/example.jar" and
        example/FontChooser.class within it.
    */
    public void testLoadClassFromPath() throws Throwable {
        String pathName = "lib/example.jar";
        String className = "example.FontChooser";
        ClassLoader cl = new PathClassLoader(pathName,
                                             getClass().getClassLoader());
        try {
            Class cls = Class.forName(className, true, cl);
        }
        catch(ClassNotFoundException cnf) {
            fail("Preloading class loader failed to load " + className
                 + " from path " + pathName);
        }
    }

    public PathClassLoaderTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, PathClassLoaderTest.class);
    }
}
