package abbot.util;

import junit.extensions.abbot.TestHelper;
import junit.framework.TestCase;

public class PreloadingClassLoaderTest extends TestCase {

    public void testNoDelegation() throws Throwable {
        String className = "test.fake.ClassForReloading";
        ClassLoader cl =
            new PreloadingClassLoader(null, getClass().getClassLoader());
        Class cls = Class.forName(className, true, cl);
        assertTrue("Class loader should define the class package (it is null)",
                   cls.getPackage() != null);
        assertEquals("Class loader defined the wrong package",
                     "test.fake", cls.getPackage().getName());

        Object obj = cls.newInstance();
        assertEquals("Wrong initial instance count on first load",
                     "1", obj.toString());

        cl = new PreloadingClassLoader(null, getClass().getClassLoader());
        cls = Class.forName(className, true, cl);
        obj = cls.newInstance();
        assertEquals("Wrong initial instance count when not delegating",
                     "1", obj.toString());
    }

    private class DelegatingClassLoader extends PreloadingClassLoader {
        public DelegatingClassLoader(String path, ClassLoader parent) {
            super(path, parent);
        }
        protected boolean shouldDelegate(String cname) { return true; }
    }

    public void testDelegation() throws Throwable {
        String className = "test.fake.ClassForReloading";
        ClassLoader parent = getClass().getClassLoader();
        ClassLoader cl = new DelegatingClassLoader("classes", parent);
        Class cls = Class.forName(className, true, cl);
        Object obj = cls.newInstance();
        assertEquals("Wrong initial instance count on first load",
                     "1", obj.toString());

        cl = new DelegatingClassLoader("classes", parent);
        cls = Class.forName(className, true, cl);
        obj = cls.newInstance();
        assertEquals("Wrong instance count when delegating",
                     "2", obj.toString());
    }

    public PreloadingClassLoaderTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, PreloadingClassLoaderTest.class);
    }
}
