
package com.ibm.abbot;

import com.ibm.abbot.tester.swt.*;
import com.ibm.abbot.script.*;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;

import junit.extensions.abbot.*;

import java.util.*;
/** 
 * Verify we can generate and look up widgets in a wide variety of GUI
 * configurations.  Note that this tests the WidgetReference as well,
 * since the two classes work closely together.
 */
public class DefaultWidgetFinderTest extends WidgetTestFixture{

	private DefaultWidgetFinder finder =
		(DefaultWidgetFinder)DefaultWidgetFinder.getFinder();
	private SimpleResolver resolver;

	public void testGetWidgetChildren(){
		Shell shell = createShell();
		Group panel = new Group(shell, SWT.HORIZONTAL);
		panel.setLayout(new RowLayout(SWT.HORIZONTAL));		
		Button button1 = new Button(panel,SWT.NONE);
		Button button2 = new Button(panel,SWT.NONE);
		button1.setText("button");
		button2.setText("button");
		//button1.setData("name","button");
		//button2.setData("name","button");
				
		showShell(shell);
		
		Widget[] children = finder.getWidgetChildren(shell,true);
		assertEquals("Failed to recursively ID all children of Shell",
					children.length,3);		
		children = finder.getWidgetChildren(shell,false);
		assertEquals("Failed to ID single child of Shell",
					children.length,1);			
		children = finder.getWidgetChildren(panel,true);
		assertEquals("Failed to recursively ID all children of Group",
					children.length,2);	
		children = finder.getWidgetChildren(panel,false);
		assertEquals("Failed to ID all children of Group",
					children.length,2);	
		hideShell(shell);
	}

	static Display display0;
	static Shell shell0,shell1;
	static boolean finished;
	static boolean started;
	public void testGetRootShells(){
		finished = false;
		shell1 = createShell();	
		showShell(shell1);	
		shell1.setText("testGetRootShells1");
		
		Thread newThread = new Thread(){
			public void run(){
				display0 = new Display();
				shell0 = new Shell(display0,SWT.SHELL_TRIM);
				shell0.setSize(200,200);
				showShell(shell0);
				shell0.setText("testGetRootShells0");
				shell0.setLocation(50,50);
				//synchronized(this){
					started = true;
				//}
				while(!finished&&!shell0.isDisposed()){
					if(!display0.readAndDispatch())
						display0.sleep();
				}
					
				hideShell(shell0);
			}
		};
		newThread.setName("New Thread");
		newThread.start();
		while(true){
			//synchronized(this){
				if(started){
					break;
				}
			//}
		}
		//shell1 = createShell();	
		//showShell(shell1);	
		//shell1.setText("testGetRootShells1");
		
				
		int count = finder.getDisplays().length;
		assertTrue("Wrong number of displays found",count==2);
		
		Shell[] shells = finder.getRootShells();
		boolean found0 = false, found1=false;
		for(int i=0; i<shells.length;i++){
			if(shells[i]==shell0)
				found0 = true;
			if(shells[i]==shell1)
				found1 = true;
		}
		
		finished = true;
		assertTrue("Shell created in this thread was not found", found1);
		assertTrue("Shell created in new thread was not found", found0);
				
		hideShell(shell1);
		
	}

	/**
	 * Test to see if the label and index can identify a component
	 */
	public void testSiblings(){
		Shell shell = createShell();
		Group panel = new Group(shell, SWT.NONE);
		panel.setLayout(new RowLayout(SWT.HORIZONTAL));
		Text tf1 = new Text(panel, SWT.NONE);
		Text tf2 = new Text(panel, SWT.NONE);
		tf1.setData("name","tf1");
		tf2.setData("name","tf2");
		panel.setData("name","pane");
		
		showShell(shell);
	
		
		WidgetReference wr1 = resolver.addWidget(tf1);
		WidgetReference wr2 = resolver.addWidget(tf2);
		assertTrue("Should get two unique references", wr1 != wr2);
		
		int w1 = finder.getWidgetMatchWeight(tf1, wr1);
		int w2 = finder.getWidgetMatchWeight(tf2, wr1);		
		assertTrue("Widget 2" + w2 + ") matched at least as well as "
				   + "Widget 1 (" + w1 + ")", w1 > w2);
		
		w1 = finder.getWidgetMatchWeight(tf1, wr2);	
		w2 = finder.getWidgetMatchWeight(tf2, wr2);	
        assertTrue("Widget 1 (" + w1 + ") matched at least as well as "
				   + "Widget 2 (" + w2 + ")", w2 > w1);
				   		
		hideShell(shell);		
	}

	/**
	 * Verify two identical sibling components can be distinguished.
	 */

	public void testIdenticalSiblings() {
		Shell shell = createShell();
		Group panel = new Group(shell, SWT.HORIZONTAL);
		panel.setLayout(new RowLayout(SWT.HORIZONTAL));		
		Button button1 = new Button(panel,SWT.NONE);
		Button button2 = new Button(panel,SWT.NONE);
		button1.setText("button");
		button2.setText("button");
		//button1.setData("name","button");
		//button2.setData("name","button");
		
		
		showShell(shell);

		WidgetReference wr1 = resolver.addWidget(button1);
		WidgetReference wr2 = resolver.addWidget(button2);
		assertTrue("Should get two unique references", wr1 != wr2);

		int w1 = finder.getWidgetMatchWeight(button1, wr1);
		int w2 = finder.getWidgetMatchWeight(button2, wr1);
        
		assertTrue("Button 2 (" + w2 + ") matched at least as well as "
				   + "Button 1 (" + w1 + ")", w1 > w2);

		w1 = finder.getWidgetMatchWeight(button1, wr2);
		w2 = finder.getWidgetMatchWeight(button2, wr2);

		assertTrue("Button 1 (" + w1 + ") matched at least as well as "
				   + "Button 2 (" + w2 + ")", w2 > w1);
				   
		hideShell(shell);
	}

    /**
     * Verify two identical cousin (sibling parents) components can be
     * distinguished. 
     */
	public void testIdenticalCousins(){
		Shell shell = createShell();
		Group panel1 = new Group(shell, SWT.HORIZONTAL);
		panel1.setLayout(new RowLayout(SWT.HORIZONTAL));
		Group panel2 = new Group(shell,SWT.HORIZONTAL);
		panel2.setLayout(new RowLayout(SWT.HORIZONTAL));		
		Button button1 = new Button(panel1,SWT.NONE);
		Button button2 = new Button(panel2,SWT.NONE);
		button1.setText("button");
		button2.setText("button");
		button1.setData("name","button");
		button2.setData("name","button");
		panel1.setData("name","panel1");
		panel2.setData("name","panel2");

		showShell(shell);

		WidgetReference cr1 = resolver.addWidget(button1);
		WidgetReference cr2 = resolver.addWidget(button2);
		assertTrue("Should get two unique references", cr1 != cr2);

		int w1 = finder.getWidgetMatchWeight(button1, cr1);
		int w2 = finder.getWidgetMatchWeight(button2, cr1);

		assertTrue("Button 2 (" + w2 + ") matched at least as well as "
				   + "Button 1 (" + w1 +")", w1 > w2);

		w1 = finder.getWidgetMatchWeight(button1, cr2);
		w2 = finder.getWidgetMatchWeight(button2, cr2);

		assertTrue("Button 1 (" + w1 + ") matched at least as well as "
				   + "Button 2 (" + w2 +")", w2 > w1);

		hideShell(shell);
	}


	/**
	 * Test to see if two buttons which are identical except for their name
	 * and index can be found
	 */
	public void testTwoNamedButtons(){
		Shell shell = createShell();
		Group pane = new Group(shell,SWT.NONE);
		pane.setLayout(new RowLayout(SWT.HORIZONTAL));
		Group panel1 = new Group(pane, SWT.NONE);
		panel1.setLayout(new RowLayout(SWT.HORIZONTAL));		
		Group panel2 = new Group(pane,SWT.NONE);
		panel2.setLayout(new RowLayout(SWT.HORIZONTAL));
		Button button1 = new Button(panel1,SWT.NONE);
		Button button2 = new Button(panel2,SWT.NONE);
		button1.setText("Button");
		button2.setText("Button");
		button1.setData("name","Button1");
		button2.setData("name","Button2");		
		
		showShell(shell);
				
		WidgetReference cr1 = resolver.addWidget(button1);
		WidgetReference cr2 = resolver.addWidget(button2);
		assertTrue("Should get two unique references", cr1 != cr2);

		int w1 = finder.getWidgetMatchWeight(button1, cr1);
		int w2 = finder.getWidgetMatchWeight(button2, cr1);

		assertTrue("Button 2 matches at least as well as Button 1",
				   w1 > w2);
		w1 = finder.getWidgetMatchWeight(button1, cr2);
		w2 = finder.getWidgetMatchWeight(button2, cr2);

		assertTrue("Button 1 matches at least as well as Button 2",
				   w2 > w1);		
				   
		hideShell(shell);		   	
	}
	

	/**
	 * Test to ensure that index can be used on otherwise indistingushable 
	 * components
	 */
	public void testIdenticalLabelGroup() {
		final int NUM_LABELS = 10;
		String text = "label";

		ArrayList labels = new ArrayList();
		ArrayList refs = new ArrayList();
		
		Shell shell = createShell();
		Group pane = new Group(shell,SWT.NONE);
		pane.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		
				
		for(int i=0; i< NUM_LABELS; i++){
			Label label = new Label(pane,SWT.CENTER);
			label.setText(text);
			labels.add(label);
		}
		
		showShell(shell);
		
		for(int i=0; i<NUM_LABELS;i++){
			Label label = (Label)(labels.get(i));
			refs.add(resolver.addWidget(label));			
		}
			
		for (int i=0;i < NUM_LABELS - 1; ++i) {
			for (int next=i+1;next < NUM_LABELS;next++) {	
				Widget w1 = (Widget)labels.get(i);
				Widget w2 = (Widget)labels.get(next);
				WidgetReference wr1 = (WidgetReference)refs.get(i);
				WidgetReference wr2 = (WidgetReference)refs.get(next);
				
				int wt1 = finder.getWidgetMatchWeight(w1, wr1);
				int wt2 = finder.getWidgetMatchWeight(w2, wr1);
				assertTrue("label " + next + " (" + wt2 + ") matches at least "
						   + "as well as label " + i + " (" + wt1 + ")",
						   wt1 > wt2);

				wt1 = finder.getWidgetMatchWeight(w1, wr2);
				wt2 = finder.getWidgetMatchWeight(w2, wr2);
				assertTrue("label " + i + " (" + wt1 + ") matches at least "
						   + "as well as label " + next + " (" + wt2 + ")",
						   wt2 > wt1);
			}
		}
		
		hideShell(shell);
	}

	/**
	 * Test to ensure that Text fields can be found.
	 */
	public void testCanFindTextField() {
		Shell shell = createShell();
		Text textField = new Text(shell, SWT.NONE);
		textField.setData("name","text");		
		
		showShell(shell);
				
		WidgetReference cr1 = resolver.addWidget(textField);
        assertNotNull("Text field not found", finder.findWidget(cr1));
		
		hideShell(shell);
	}

	/**
	  * Test that unamed widgets have the correct default value for 
	  * their name
	  */
	 public void testGetWidgetName() {
	 	Shell shell = createShell();
		shell.setText("Shell");
		Widget widget = new Button(shell,SWT.NONE);
				
		assertNull("Unnamed Widget's distilled name should be null",
					finder.getWidgetName(widget));
		widget.setData("name","button");
		assertEquals("Named Widget's name is used",
					  (String)(widget.getData("name")), finder.getWidgetName(widget));

		assertNull("Unnamed Shell's distilled name should be null",
					finder.getWidgetName(shell));
	  
	  	hideShell(shell);
	  }
	  
	public void testFiltering(){
		Shell shell = createShell();
		Group pane = new Group(shell,SWT.NONE);
		pane.setLayout(new RowLayout(SWT.HORIZONTAL));
		Group panel1 = new Group(pane, SWT.NONE);
		panel1.setLayout(new RowLayout(SWT.HORIZONTAL));		
		Group panel2 = new Group(pane,SWT.NONE);
		panel2.setLayout(new RowLayout(SWT.HORIZONTAL));
		Button button1 = new Button(panel1,SWT.NONE);
		Button button2 = new Button(panel2,SWT.NONE);
		button1.setText("Button");
		button2.setText("Button");
		button1.setData("name","Button1");
		button2.setData("name","Button2");		
		
		showShell(shell);
		
		WidgetReference wrButton1 = resolver.addWidget(button1);
		
		Widget found = finder.findWidget(wrButton1);
		assertEquals("Couldn't find button1",button1,found);
		finder.filterWidget(button1);
		found = null;
		try{
			found = finder.findWidget(wrButton1);		
		}
		catch(WidgetNotFoundException wnfe){
		}
		finally{
			assertTrue("Failed to filter button1",found==null);
			hideShell(shell);		
		}
	}
	
	public DefaultWidgetFinderTest(String name) {
		super(name);
	}
	protected void setUp() {
		resolver = new SimpleResolver();
	}
	public static void main(String[] args) {
		TestHelper.runTests(args, DefaultWidgetFinderTest.class);
	}

}
