
package com.ibm.abbot.editor.recorder;

import com.ibm.abbot.tester.swt.*;
import com.ibm.abbot.*;
import com.ibm.abbot.script.*;

import junit.framework.*;

import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;

import com.ibm.abbot.examples.CelsiusConverter;

/** 
 * Unit tests for class WidgetRecorder and EventRecorder that test 
 * BASIC recording functionality- ie, that the method name in the generated
 * Action step is correct. This does not test playback of recorded events.
 */
public class WidgetRecorderTest extends TestCase{
	static EventRecorder recorder;
	static WidgetTester tester;
	static Thread appThread;
	static boolean startedApp = false;
	static Shell shell;
	
		
	public void setUp(){
		// start GUI in another thread 
		if(!startedApp){
			appThread = new Thread(){
				public void run(){
				   CelsiusConverter.main(null);
			   }
			};
			appThread.setName("AppThread");
			appThread.start();	
		
			//startedApp = true;
		}		
						
		// make sure GUI is showing
		tester = WidgetTester.getTester(Shell.class);
		WidgetTester.waitForFrameShowing("Convert Celsius to Fahrenheit");
		WidgetReference ref = new WidgetReference(null,Shell.class,
									null,null,null,"Convert Celsius to Fahrenheit");
		shell = (Shell)DefaultWidgetFinder.getFinder().findWidget(ref);
		
		tester.actionFocus(shell);
		recorder = 	new EventRecorder(new SimpleResolver(),shell.getDisplay(),true	);
		recorder.start();		

	}
	public void tearDown(){
		recorder.terminate();
		Robot.syncExec(shell.getDisplay(),null,new Runnable(){
			public void run(){
				shell.close();
				shell.getDisplay().dispose();
			}				
		});			
	}
	
	public void testRecordClick(){
		WidgetReference ref = new WidgetReference("Convert", Button.class,"Convert");
		Button convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);
		
		tester.actionClick(convertButton);
		tester.actionWaitForIdle(convertButton.getDisplay());
		
		Step steps = recorder.getStep();
		try{
			assertTrue("Failed to record actionClick",
				((Call)((Sequence)steps).getStep(0)).getMethodName().equals("actionClick"));
		}
		catch(ClassCastException cce){
			fail("Failed to record actionClick");
		}
	}

	public void testRecordKey(){
		WidgetReference ref = new WidgetReference(null, Text.class,null);
		Text text = (Text)DefaultWidgetFinder.getFinder().findWidget(ref);
		tester = WidgetTester.getTester(Text.class);
		((TextTester)tester).actionKey('A',text.getDisplay());
		((TextTester)tester).actionKey('\r',text.getDisplay());
		
		Step steps = recorder.getStep();
		try{
			assertTrue("Failed to record actionKey",
				((Call)((Sequence)steps).getStep(0)).getMethodName().equals("actionKey"));
		}
		catch(Exception e){
			fail("Failed to record actionKey");
		}
		assertTrue("Failed to record all actionKey actions (2)",
			((Sequence)steps).size()==2);
	}



	public void testRecordMenuSelection(){
		WidgetReference ref = new WidgetReference("precision3",MenuItem.class,"2");
		MenuItem item2 = (MenuItem)DefaultWidgetFinder.getFinder().findWidget(ref);
	
		tester = WidgetTester.getTester(Widget.class);
		tester.actionSelectMenuItem(item2);
		
		Step steps = recorder.getStep();
		
		try{
			assertTrue("Failed to record actionSelectMenuItem",
				((Call)((Sequence)steps).getStep(0)).getMethodName().equals("actionSelectMenuItem"));
		}
		catch(Exception e){
			fail("Failed to record actionSelectMenuItem");
		}
	}

	public void testRecordDoubleClick(){
		WidgetReference ref = new WidgetReference("fahrenheitLabel",Label.class,"fahrenheitLabel");
		Label fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);	
		
		tester = WidgetTester.getTester(Label.class);
		tester.actionClick(fahrenheitLabel,5,5,"BUTTON1",2);
		
		Step steps = recorder.getStep();
		try{
			Action clickStep = ((Action)((Sequence)steps).getStep(0));
			int numClicks = Integer.parseInt(clickStep.getArgs()[4]);
			assertTrue("Failed to record actionClick",clickStep.getMethodName().equals("actionClick"));
			assertTrue("Failed to record a double-click actionClick",numClicks==2);
			
		}
		catch(ClassCastException cce){
			fail("Failed to record actionClick");
		}
			
	}

	public void testRecordClickWithModifiers(){
		WidgetReference ref = new WidgetReference("celsiusLabel",Label.class,"celsiusLabel");
		Label celsiusLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);	
		
		tester = WidgetTester.getTester(Label.class);
		tester.actionClick(celsiusLabel,5,5,"BUTTON1 | SHIFT",2);
		
		Step steps = recorder.getStep();
		try{
			Action clickStep = ((Action)((Sequence)steps).getStep(0));
			int numClicks = Integer.parseInt(clickStep.getArgs()[4]);
			int modifiers = Integer.parseInt(clickStep.getArgs()[3]);
			assertTrue("Failed to record actionClick",clickStep.getMethodName().equals("actionClick"));
			assertTrue("Failed to record a double-click actionClick",numClicks==2);
			assertTrue("Failed to record modifiers for actionClick",
						modifiers==(SWT.BUTTON1|SWT.SHIFT));
			
		}
		catch(ClassCastException cce){
			fail("Failed to record actionClick");
		}		
	}
	
	// not currently supported
	public void testRecordWidgetNotVisible(){
				
	}

	Shell shell2;
	public void testRecordShellShow(){
		Robot.syncExec(shell.getDisplay(),null,new Runnable(){
			public void run(){
				shell2 = new Shell(shell.getDisplay(),SWT.SHELL_TRIM);
				shell2.setText(getName());
				shell2.setSize(200,200);
				shell2.open();	
			}
		});
		
		Step steps = recorder.getStep();
		try{
			com.ibm.abbot.script.Assert step= (com.ibm.abbot.script.Assert)((Sequence)steps).getStep(0);
			String windowName = step.getArgs()[0];
			String methodName = step.getMethodName();
			assertTrue("Failed to record assertDecorationsShowing assertion",methodName.equals("assertDecorationsShowing"));
			assertTrue("Failed to record the correct Shell name",windowName.equals(getName()));
			
		}catch(Exception e){
			e.printStackTrace();
			fail("Failed to record assertDecorationsShowing assertion");
		}finally{
			Robot.syncExec(shell.getDisplay(),null,new Runnable(){
				public void run(){
					shell2.close();
				}
			});		
		}
	}
	
	public void testRecordControlKey(){
		WidgetReference ref = new WidgetReference(null, Text.class,null);
		Text text = (Text)DefaultWidgetFinder.getFinder().findWidget(ref);
		tester = WidgetTester.getTester(Text.class);
		((TextTester)tester).actionKey('a'|SWT.CONTROL,text.getDisplay());
		
		Step steps = recorder.getStep();
		assertTrue("Failed to omit control-key typed", 
					((Sequence)steps).size()==0);			
	}
	
	public void testRecordAltKey(){
		WidgetReference ref = new WidgetReference(null, Text.class,null);
		Text text = (Text)DefaultWidgetFinder.getFinder().findWidget(ref);
		tester = WidgetTester.getTester(Text.class);
		((TextTester)tester).actionKey('b'|SWT.ALT,text.getDisplay());
		
		Step steps = recorder.getStep();
		assertTrue("Failed to omit control-key typed", 
					((Sequence)steps).size()==0);		  
	}
	
	Text text;
	public void testRecordSpecialKeys(){
		// open a separate multi-line text window so we can do things like TAB where it 
		// will generate a key event instead of tabbing among controls
		Robot.syncExec(shell.getDisplay(),null,new Runnable(){
			public void run(){
				shell2 = new Shell(shell.getDisplay(),SWT.SHELL_TRIM);
				shell2.setText(getName());
				text = new Text(shell2, SWT.MULTI|SWT.WRAP);
				text.setLocation(0,0);
				text.setSize(200,200);

				shell2.pack();				
				shell2.open();	
			}
		});
		tester.actionFocus(text);
		char[] keys = {SWT.CR, SWT.TAB,SWT.BS, SWT.DEL, SWT.ESC};
		String[] keyNames = {"SWT.CR","SWT.TAB","SWT.BS","SWT.DEL","SWT.ESC"};
		int i=0;
			for (i=0;i < keys.length;i++) {
			tester.actionKey(keys[i],shell.getDisplay());

			Step steps = recorder.getStep();
			try{
				Action step = (Action)((Sequence)steps).getStep(i+1);
				char keyTyped = (char)(Integer.parseInt(step.getArgs()[0]));
				String methodName = step.getMethodName();
				assertTrue("Failed to record actionKey",
							methodName.equals("actionKey"));
				assertTrue("Failed to record keyStroke for "+keyNames[i],
							keyTyped==keys[i]);
				
			}catch(Exception e){
				fail("Failed to record keyStroke for "+keyNames[i]);
			}
			
		}
		
		Robot.syncExec(shell2.getDisplay(),null,new Runnable(){
			public void run(){
				shell2.close();
			}
		});
	}

	public void testRecordPossibleUserSequence(){
		WidgetReference ref = new WidgetReference("precision2",MenuItem.class,"2");
		MenuItem item2 = (MenuItem)DefaultWidgetFinder.getFinder().findWidget(ref);
		ref = new WidgetReference("precision3",MenuItem.class,"3");
		MenuItem item3 = (MenuItem)DefaultWidgetFinder.getFinder().findWidget(ref);
		ref = new WidgetReference("tempCelsius", Text.class,"tempCelsius");
		Text tempCelsius = (Text)DefaultWidgetFinder.getFinder().findWidget(ref); 
		ref = new WidgetReference("Convert", Button.class,"Convert");
		Button convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);			
		
		// change the precision, enter a value, click the 'convert' button, 
		// minimize the shell, restore it, change the precision, delete the old value,
		// enter a new value, and click 'convert' again 
		tester.actionSelectMenuItem(item2);
		tester.actionFocus(tempCelsius);
		tester.actionKeyString("23.4\r",shell.getDisplay());
		tester.actionSelectMenuItem(item3);
		tester.actionKey(SWT.BS,shell.getDisplay());
		tester.actionKey(SWT.BS,shell.getDisplay());
		tester.actionKey(SWT.BS,shell.getDisplay());
		tester.actionKey(SWT.BS,shell.getDisplay());
		tester.actionKeyString("-12.2\r",shell.getDisplay());
		
		Sequence steps = (Sequence)recorder.getStep();
		
		String methodName;
		for(int i=0; i<steps.size();i++){
			methodName =((Action)steps.getStep(i)).getMethodName();
			if(i==0 || i==6)
				assertTrue("Failed to record actionSelectMenuItem",
							methodName.equals("actionSelectMenuItem"));				
			else 
				assertTrue("Failed to record actionKey",
							methodName.equals("actionKey"));
		}	
	}

//	public void testRecordDND(){
//		WidgetReference ref1 = new WidgetReference("celsiusLabel",Label.class,"celsiusLabel");
//		Label celsiusLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref1);	
//		WidgetReference ref2 = new WidgetReference("fahrenheitLabel",Label.class,"fahrenheitLabel");
//		Label fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref2);			
//
//		tester = WidgetTester.getTester(Label.class);
//		tester.actionDrag(celsiusLabel,2,2);
//		tester.actionDrop(fahrenheitLabel,4,4);	  
//		
//		Step steps = recorder.getStep();		
//				
//	}
}

