
package com.ibm.abbot.examples;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.*;

/**
 * This is the sample app for the demo described at 
 * <url>http://www.sys-con.com/java/articleprint.cfm?id=1940</url> that
 * provides a means to convert Celsius temperatures to Fahrenheit.
 */
public class CelsiusConverter{
	public Shell shell;
	public Button convertButton;
	public Label fahrenheitLabel,celsiusLabel;
	public Text tempCelsius;
	MenuItem[] precisionVal;
	
	double fTemp = 32;
	double cTemp = 0;
	static int precision;
	
	/** Create the GUI and open it **/
	public CelsiusConverter(Display display){
		// setup the gui
		shell = new Shell(display,SWT.SHELL_TRIM|SWT.NO_REDRAW_RESIZE);
		shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		((RowLayout)shell.getLayout()).pack = false;
		shell.setText("Convert Celsius to Fahrenheit");
		shell.setSize(300,110);
		
		Menu bar = createBarMenu();
		
		Group leftGroup = new Group(shell,SWT.CENTER);
		leftGroup.setData("name","leftGroup");
		leftGroup.setLayout(new RowLayout(SWT.VERTICAL));
		((RowLayout)leftGroup.getLayout()).pack = false;
		
		tempCelsius = new Text(leftGroup,SWT.BORDER|SWT.CENTER/*|SWT.WRAP*/);
		tempCelsius.setData("name","tempCelsius");
		
		convertButton = new Button(leftGroup,SWT.PUSH);
		convertButton.setText("Convert");
		convertButton.setData("name","Convert");				
		convertButton.setSize(200,40);
		
		Group rightGroup = new Group(shell,SWT.LEFT);
		rightGroup.setLayout(new RowLayout(SWT.VERTICAL));		
		((RowLayout)rightGroup.getLayout()).pack = false;
		rightGroup.setData("name","rightGroup");
				
		fahrenheitLabel = new Label(rightGroup,SWT.NONE);
		fahrenheitLabel.setText("Fahrenheit: \t"+format(fTemp)+"\u00B0      ");
		fahrenheitLabel.setData("name","fahrenheitLabel");

		celsiusLabel = new Label(rightGroup,SWT.NONE);
		celsiusLabel.setText("Celsius: \t"+format(cTemp)+"\u00B0        ");
		celsiusLabel.setData("name","celsiusLabel");
	
		// add listeners
		convertButton.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event e){
				setTemps();			
			}
			
		});	
		tempCelsius.addListener(SWT.KeyUp, new Listener(){
			public void handleEvent(Event e){
				if((e.keyCode&SWT.CR)==SWT.CR){
					setTemps();
				}
			}
		});
		shell.pack();	
		shell.setLocation(50,50);		
		// open the shell	
		shell.open();
		shell.setMinimized(false);
	}
	
	public static String format(double d){
		if(d==0){
			String res = ""+d;
			for(int i=0; i<precision-1;i++)
				res+="0";
			return res;
		}
			
		String sign = (d<0)?"-":"";
		d = Math.abs(d);
		if(precision==0){
			return sign+Math.round(d);
		}
		d = d*Math.pow(10,precision);
		long digits = (long)Math.round(d);
		String val = Long.toString(digits);
		int leftOfDeci = val.length()-precision;
		
		return ""+sign+val.substring(0,leftOfDeci)+"."+val.substring(leftOfDeci);
	}
	
//	public static String format(double d){
//		String sign = (d<0)?"-":"";
//		d = Math.abs(d);
//		int leftOfDecimal = (int)d;
//		
//		int rightOfDecimal = (int)((d-leftOfDecimal)*Math.pow(10,precision));
//		String rightOfDecimalString = ""+rightOfDecimal;
//		while(rightOfDecimalString.length()<precision)
//			rightOfDecimalString="0"+rightOfDecimalString;		
//		
//		return (precision==0)? (sign+leftOfDecimal) : (sign+leftOfDecimal+"."+rightOfDecimalString);
//	}
	
	public void setTemps(){
		try{
			cTemp = Double.parseDouble(tempCelsius.getText());
		}
		catch(Exception ignored){
		}
		fTemp = 9*cTemp/5 + 32;
		celsiusLabel.setText("Celsius: \t"+format(cTemp)+"\u00B0");
		fahrenheitLabel.setText("Fahrenheit: \t"+format(fTemp)+"\u00B0");
	}
	
	public Menu createBarMenu(){
		Menu bar = new Menu(shell, SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem optionsItem = new MenuItem(bar,SWT.CASCADE);
		optionsItem.setText("Options");
		Menu optionsMenu = new Menu(shell,SWT.DROP_DOWN);
		optionsItem.setMenu(optionsMenu);
		MenuItem precisionItem = new MenuItem(optionsMenu,SWT.CASCADE);		
		//MenuItem precisionItem = new MenuItem(bar,SWT.CASCADE);//
		precisionItem.setText("Precision");
		Menu precisionMenu = new Menu(shell,SWT.DROP_DOWN);
		precisionItem.setMenu(precisionMenu);
		precisionVal = new MenuItem[5];
		for(int i=0; i<5;i++){
			precisionVal[i] = new MenuItem(precisionMenu,SWT.CHECK);
			precisionVal[i].setText(""+i);
			precisionVal[i].setData("name",""+i);
		}
		precisionVal[0].setSelection(true);
		precision = 0;		
		
		Listener precisionListener = new Listener(){
			public void handleEvent(Event e){
				// clear the old precision item
				precisionVal[precision].setSelection(false);
				
				// set new precision	
				for(int i=0; i<5;i++){
					if(precisionVal[i].getSelection()){
						precision = i;
						break;
					}					
				}
				
				// draw labels
				setTemps();
			}
		};
		
		for(int i=0; i<5;i++){
			precisionVal[i].addListener(SWT.Selection, precisionListener);
		}
		
		return bar;
	}
	
	/** init the app and start the event-loop **/	
	public static void main(String[] argv){
		Display display = new Display();
		
		CelsiusConverter converter = new CelsiusConverter(display);
		
		while(!converter.shell.isDisposed()){
			if(!display.readAndDispatch())
				display.sleep();
		}	
		
		display.dispose();	
	}
}
