
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;

public class ComboTesterTest extends WidgetTestFixture{
	ComboTester tester;
	
	public ComboTesterTest(String name){
		super(name);
		tester = (ComboTester)WidgetTester.getTester(Combo.class);		
	}
	
	public void testGetContentsAndGetItem(){
		Shell shell = createShell();
		Combo combo = new Combo(shell,SWT.SIMPLE);		
		String[] entries = new String[10];
		for(int i=0; i<entries.length;i++){
			entries[i]="entry"+i;
			combo.add(entries[i]);
		}
		int j=0;
		showShell(shell);
		String[] contents = tester.getItems(combo);
		
		assertEquals("contents array not proper length",entries.length,
					 contents.length);
		for(int i=0; i<entries.length;i++){
			assertTrue("entry "+i+" does not match(GetContents)",
					entries[i].equals(contents[i]));
			assertTrue("entry "+i+" does not match(GetItem)",
					entries[i].equals(tester.getItem(combo,i)));			 
		}	
		
		hideShell(shell);	
	}
	
	public void testActionSelectIndex(){
		Shell shell = createShell();
		Combo combo = new Combo(shell,SWT.DROP_DOWN);		
		String[] entries = new String[10];
		for(int i=0; i<entries.length;i++){
			entries[i]="entry"+i;
			combo.add(entries[i]);
		}
	
		showShell(shell);
		int index;
		
		for(int i=0; i<entries.length*2;i++){
			index = (int)(entries.length*Math.random());
			tester.actionSelectIndex(combo,index);
			assertEquals("entry "+index+" is not selected",
						index,combo.getSelectionIndex());
		}
		
		hideShell(shell);
	}
	
	public void testActionSelectItem(){
		Shell shell = createShell();
		Combo combo = new Combo(shell,SWT.DROP_DOWN);		
		String[] entries = new String[10];
		for(int i=0; i<entries.length;i++){
			entries[i]="entry"+i;
			combo.add(entries[i]);
		}
	
		showShell(shell);
		int index;
		
		for(int i=0; i<entries.length*2;i++){
			index = (int)(entries.length*Math.random());
			tester.actionSelectItem(combo,entries[index]);
			assertTrue("entry \""+entries[index]+"\" is not selected",
						combo.getItem(combo.getSelectionIndex()).equals(entries[index]));
		}
		
		hideShell(shell);		
	}
	
	public void getSelectedItem(){
		Shell shell = createShell();
		Combo combo = new Combo(shell,SWT.DROP_DOWN);		
		String[] entries = new String[10];
		for(int i=0; i<entries.length;i++){
			entries[i]="entry"+i;
			combo.add(entries[i]);
		}
	
		showShell(shell);
		
		for(int i=0; i<combo.getItemCount();i++){
			assertTrue("selected item not identified correctly",
						combo.getItem(combo.getSelectionIndex()).equals(tester.getSelection(combo)));
		}
			
		hideShell(shell);					
	}
	
	public void testAssertions(){
		Shell shell = createShell();
		Combo combo = new Combo(shell,SWT.DROP_DOWN);		
		String[] entries = new String[10];
		for(int i=0; i<entries.length;i++){
			entries[i]="entry"+i;
			combo.add(entries[i]);
		}
	
		showShell(shell);
		
		String selected;
		for(int i=0; i<combo.getItemCount();i++){
			combo.select(i);
			selected = combo.getItem(combo.getSelectionIndex());
			for(int j=0; j<combo.getItemCount();j++){
				assertEquals("assertion incorrect (index)",
							(i==j),tester.assertIndexSelected(combo,j));							
				assertEquals("assertion incorrect (item)",
							 (i==j),tester.assertItemSelected(combo,combo.getItem(j)));
			}			
		}
			
		hideShell(shell);				
	}

//	public void testClick(){
//		Shell shell = createShell();
//		Combo combo = new Combo(shell,SWT.SIMPLE);		
//		String[] entries = new String[10];
//		for(int i=0; i<entries.length;i++){
//			entries[i]="entry"+i;
//			combo.add(entries[i]);
//		}	
//		
//		showShell(shell);
//		tester.delay(1000);
//		
//		
//		hideShell(shell);	
//	}
}	
