
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.events.*;

/**
 * Tests the SashTester class.  NOTE: All of the code for 
 * creatingt the example GUI is taken directly from 
 * org.eclipse.swt.examples.controlexample.SashTab.java
 */
public class SashTesterTest extends WidgetTestFixture{
	Shell shell;
	Sash hSash, vSash;
	List list1, list2, list3;
	Text text;
	Composite content;

	static SashTester tester =(SashTester)WidgetTester.getTester(Sash.class);

	// use the SWT Examples that come with Eclipse for testing
	
	public void setUp(){
		shell = createShell();
		content = createContent(shell);
		content.setSize(300,300);
		shell.pack();
		showShell(shell);
	}
	
	public void tearDown(){
		hideShell(shell);
	}
	/* Example widgets and groups that contain them */


	static String [] ListData0 = {"Kevin","Jon","Stephen","Bill","Bonnie"};
								  
	static String [] ListData1 = {"Cheese","Beef","Chicken","Broth","Milk"};

	/* Constants */
	static final int SASH_WIDTH = 3;

	/**
	 * Creates the tab folder page.
	 */
	Composite createContent(Shell shell) {
		/*
		 * Create the page.  This example does not use layouts.
		 */
		content = new Composite(shell, SWT.BORDER);
	
		/* Create the list and text widgets */
		list1 = new List (content , SWT.V_SCROLL | SWT.H_SCROLL | SWT.BORDER);
		list1.setItems (ListData0);
		list2 = new List (content , SWT.V_SCROLL | SWT.H_SCROLL | SWT.BORDER);
		list2.setItems (ListData1);
		text = new Text (content , SWT.MULTI | SWT.BORDER);
		text.setText ("Text goes here");
	
		/* Create the sashes */
		vSash = new Sash (content , SWT.VERTICAL);
		hSash = new Sash (content , SWT.HORIZONTAL);
		
		/* Add the listeners */
		hSash.addSelectionListener (new SelectionAdapter () {
			public void widgetSelected (SelectionEvent event) {
				if (event.detail != SWT.DRAG) {
					hSash.setBounds (event.x, event.y, event.width, event.height);
					layout ();
				}
			}
		});
		vSash.addSelectionListener (new SelectionAdapter () {
			public void widgetSelected (SelectionEvent event) {
				if (event.detail != SWT.DRAG) {
					vSash.setBounds(event.x, event.y, event.width, event.height);
					layout ();
				}
			}
		});
		content.addControlListener (new ControlAdapter () {
			public void controlResized (ControlEvent event) {
				shellResized ();
			}
		});
	
		/*
		* Do not set the bounds of the lists, text and sashes here 
		* because this method is run before the widget is opened 
		* so we do not know how big the tabComposite is going to be.
		* When the widget is opened a resize event will occur 
		* and the contained widgets can be sized accordingly.
		*/
		return content; 
	}
	
	/**
	 * Gets the "Example" widget children.
	 */
	Control [] getExampleWidgets () {
		return new Control [] {hSash, vSash};
	}
	
	/**
	 * Gets the text for the tab folder item.
	 */
	String getTabText () {
		return "Sash";
	}
	
	/**
	 * Layout the list and text widgets according to the new
	 * positions of the sashes..events.SelectionEvent
	 */
	void layout () {
		
		Rectangle tabCompositeBounds = content.getClientArea ();
		Rectangle hSashBounds = hSash.getBounds ();
		Rectangle vSashBounds = vSash.getBounds ();
		
		list1.setBounds (0, 0, vSashBounds.x, hSashBounds.y);
		list2.setBounds (vSashBounds.x + vSashBounds.width, 0, tabCompositeBounds.width - (vSashBounds.x + vSashBounds.width), hSashBounds.y);
		text.setBounds (0, hSashBounds.y + hSashBounds.height, tabCompositeBounds.width, tabCompositeBounds.height - (hSashBounds.y + hSashBounds.height));
	
		/**
		* If the horizontal sash has been moved then the vertical
		* sash is either too long or too short and its size must
		* be adjusted.
		*/
		vSashBounds.height = hSashBounds.y;
		vSash.setBounds (vSashBounds);
	}
	
	/**
	 * Recreates the "Example" widgets.
	 */
	void recreateExampleWidgets () {
		// this tab does not use this framework mechanism
	}
	
	/**
	 * Handle the shell resized event.
	 */
	void shellResized () {
	
		/* Get the client area for the shell */
		Rectangle tabFolderPageBounds = content.getClientArea ();
		
		/*
		* Make list 1 half the width and half the height of the tab leaving room for the sash.
		* Place list 1 in the top left quadrant of the tab.
		*/
		Rectangle list1Bounds = new Rectangle (0, 0, (tabFolderPageBounds.width - SASH_WIDTH) / 2, (tabFolderPageBounds.height - SASH_WIDTH) / 2);
		list1.setBounds (list1Bounds);
	
		/*
		* Make list 2 half the width and half the height of the tab leaving room for the sash.
		* Place list 2 in the top right quadrant of the tab.
		*/
		list2.setBounds (list1Bounds.width + SASH_WIDTH, 0, tabFolderPageBounds.width - (list1Bounds.width + SASH_WIDTH), list1Bounds.height);
	
		/*
		* Make the text area the full width and half the height of the tab leaving room for the sash.
		* Place the text area in the bottom half of the tab.
		*/
		text.setBounds (0, list1Bounds.height + SASH_WIDTH, tabFolderPageBounds.width, tabFolderPageBounds.height - (list1Bounds.height + SASH_WIDTH));
	
		/* Position the sashes */
		vSash.setBounds (list1Bounds.width, 0, SASH_WIDTH, list1Bounds.height);
		hSash.setBounds (0, list1Bounds.height, tabFolderPageBounds.width, SASH_WIDTH);
	}
	
	public void testSetLocation(){
		tester.actionSetSashLocation(hSash,30);
		Point location = tester.getLocation(hSash);
		assertEquals("Failed to move horizontal sash",30,location.y);	
	
		tester.actionSetSashLocation(vSash,46);
		location = tester.getLocation(vSash);
		assertEquals("Failed to move vertical sash",46,location.x);
		
	}
	
	public void testMoveBy(){
		Point start = tester.getGlobalLocation(hSash);
		tester.actionMoveSashBy(hSash,55);
		Point end = tester.getGlobalLocation(hSash);
		assertEquals("Failed to move horizontal sash",start.y+55,end.y);
	
		start = tester.getGlobalLocation(vSash);
		tester.actionMoveSashBy(vSash,-20);
		end = tester.getGlobalLocation(vSash);
		assertEquals("Failed to move vertical sash",start.x-20,end.x);
	}
	
	public void testExtremeValues(){
		tester.actionMoveSashBy(hSash,50000);
		Point location = tester.getLocation(hSash);
		int parentHeight = tester.getBounds(tester.getParent(hSash)).height;
		
		assertEquals("Failed to move horizontal sash",location.y,
			parentHeight-2*tester.getBorderWidth(content)-tester.getBounds(hSash).height);
		
		tester.actionSetSashLocation(vSash,-20);
		location = tester.getLocation(vSash);
		assertEquals("Failed to move vertical sash",location.x,0);
	
	
	}
}
