
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;

public class WidgetLocatorTest extends WidgetTestFixture{
	static boolean condition = false;
	
	public WidgetLocatorTest(String name) {
		super(name);
	}

	
	public void testShell(){
		Shell shell = createShell();
		shell.addListener(SWT.MouseDown, new Listener(){
			public void handleEvent(Event e){
				condition = true;
			}
		});
	
		showShell(shell);
		
		Point loc = WidgetLocator.getLocation(shell,true);
		condition = false;
		robot.mouseMove(loc.x,loc.y);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Failed to click top-left corner of shell", condition);
		
		Rectangle bounds = WidgetLocator.getBounds(shell,true);
		condition = false;
		robot.mouseMove(bounds.x+bounds.width,bounds.y+bounds.height);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Failed to click bottom-right corner of shell", condition);
	
		hideShell(shell);
	}
	
	public void testDecorationsWithNoTrim(){
		Shell shell = createShell();
		shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		Decorations dec = new Decorations(shell,SWT.NONE);
		dec.setBackground(new Color(shell.getDisplay(),255,255,255));
		
		dec.addListener(SWT.MouseDown, new Listener(){
			public void handleEvent(Event e){
				condition = true;
			}
		});		
		
		showShell(shell);

		Point loc = WidgetLocator.getLocation(dec,true);
		condition = false;
		robot.mouseMove(loc.x,loc.y);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Failed to click top-left corner of decoration", condition);

		Rectangle bounds = WidgetLocator.getBounds(dec,true);
		condition = false;
		robot.mouseMove(bounds.x+bounds.width,bounds.y+bounds.height);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Failed to click bottom-right corner of decoration", condition);

		hideShell(shell);
	}
	
	public void testButton(){
		Shell shell = createShell();
		shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		Button button = new Button(shell,SWT.CENTER);
		
		button.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event e){
				condition = true;
			}
		});
		
		showShell(shell);

		Rectangle bounds = WidgetLocator.getBounds(button,true);
		condition = false;
		robot.mouseMove(bounds.x,bounds.y);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Failed to click top-left corner of button", condition);
	
		condition = false;
		robot.mouseMove(bounds.x+bounds.width,bounds.y+bounds.height);
		robot.mousePress(SWT.BUTTON1);
		robot.mouseRelease(SWT.BUTTON1);
		Robot.waitForIdle(shell.getDisplay());
	
		assertTrue("Failed to click bottom-right corner of button", condition);
		
		hideShell(shell);
	}	
}
