package com.ibm.swt;

import junit.framework.*;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;


import java.awt.AWTException;

/** 
 * Tests the functionality of <code>abbot.swt.Robot</code> as it
 * compares to that of <code>java.awt.Robot</code>
 */
public class RobotTest extends TestCase{
	
	Display display;
	Robot swtRobot;
	java.awt.Robot awtRobot;
	boolean testCondition;
	
	public void setUp(){
	 	display = new Display();
		swtRobot = new Robot(display); 	
		try{
			awtRobot = new java.awt.Robot();
		}
		catch(AWTException awte){
			System.err.println("Unable to create java.awt.Robot");
			awte.printStackTrace();
		}
	}

	public void tearDown(){
		display.dispose();
	}

	public void testMouseActions(){
		Shell shell = new Shell(display, SWT.SHELL_TRIM);
		shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		Button button = new Button(shell,SWT.PUSH);
		button.setSize(100,100);
		button.addListener(SWT.Selection,new Listener(){
			public void handleEvent(Event e){
				testCondition = true;
			}
		});		
		shell.setBounds(0,0, 400,400);				
		shell.open();
		Point p = button.getLocation();
		p = shell.toDisplay(p.x,p.y);
		System.out.println(p);
		
		// test 1
		String moveTestString = "mouseMove failed";
		swtRobot.mouseMove(p.x,p.y);
		Point p2 = display.getCursorLocation();
		assertEquals(moveTestString + " on X.", new Integer(p.x),new Integer(p2.x));
		assertEquals(moveTestString + " on Y.", new Integer(p.y),new Integer(p2.y));
		
		// test 2
		String clickTestString = "mousePress/mouseRelease failed";	
		testCondition = false;
		swtRobot.mousePress(SWT.BUTTON1);
		swtRobot.mouseRelease(SWT.BUTTON1);
		swtRobot.waitForIdle();
		assertTrue(clickTestString, testCondition);
	
		// dispose of shell
		shell.dispose();
	}
		
	public void testKeyboardActions(){
		int accelerator =SWT.CTRL|'a'; 
		Shell shell = new Shell(display, SWT.SHELL_TRIM);
		shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		Menu bar = new Menu(shell,SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem testItem = new MenuItem(bar,SWT.CASCADE);
		testItem.setAccelerator(accelerator);
		testItem.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event e){
				testCondition = true;
			}
		});				
		shell.open();	
	
		// test 1- type a menuItem accelerator
		String accelTest = "keyPress/keyRelease failed";
		testCondition = false;
		swtRobot.keyPress(accelerator);
		swtRobot.keyRelease(accelerator);
		swtRobot.waitForIdle();
		assertTrue(accelTest, testCondition);
						
		shell.dispose();
	}

	public void testCaptureActions(){
		// test 1- getPixelColor
		String colorTest = "getPixelColor failed";
		int x = 100;
		int y = 100;
		Color swtColor = swtRobot.getPixelColor(x,y);
		java.awt.Color awtColor = awtRobot.getPixelColor(x,y);
		assertEquals(colorTest,swtColor.getRed(),awtColor.getRed());
		assertEquals(colorTest,swtColor.getGreen(),awtColor.getGreen());
		assertEquals(colorTest,swtColor.getBlue(),awtColor.getBlue());	
	}
}
