package test;

import java.awt.Component;
import java.awt.event.*;
import javax.swing.*;

import junit.extensions.abbot.*;
import abbot.tester.awt.*;

/** Unit test to show bugs with Screen Menu Bar on OSX.  Don't include this in
 * the full test suite, since we know it fails due to Apple VM bugs.
 */
public class OSXTest extends ComponentTestFixture {

    private ComponentTester tester =
        ComponentTester.getTester(Component.class);

    private static final boolean isOSX = 
        System.getProperty("os.name").startsWith("Mac");

    private class KeyWatcher extends KeyAdapter {
        public int code = KeyEvent.VK_UNDEFINED;
        public int modifiers = 0;
        public boolean gotPress = false;
        public boolean gotRelease = false;
        public void keyPressed(KeyEvent ke) {
            gotPress = true;
            code = ke.getKeyCode();
            modifiers = ke.getModifiers();
        }
        public void keyReleased(KeyEvent ke) {
            gotRelease = true;
        }
    }

    public void testAltGraph() {
        JTextField tf = new JTextField();
        tf.setColumns(10);
        KeyWatcher kw = new KeyWatcher();
        tf.addKeyListener(kw);
        showFrame(tf);
        tester.actionFocus(tf);
        int code = KeyEvent.VK_ALT_GRAPH;
        int mods = 0;
        tester.actionKeyStroke("VK_ALT_GRAPH", "0");
        assertTrue("Never received key press", kw.gotPress);
        assertTrue("Never received release", kw.gotRelease);
        assertEquals("Wrong key code", code, kw.code);
        assertEquals("Wrong modifiers", tester.getKeyModifiers(mods), 
                     tester.getKeyModifiers(kw.modifiers));
    }

    /** Robot won't generate an ESC on OSX 1.3.1 (Apple VM bug). */
    public void testEscapeKeyStroke() {
        JTextField tf = new JTextField();
        tf.setColumns(10);
        KeyWatcher kw = new KeyWatcher();
        tf.addKeyListener(kw);
        showFrame(tf);
        tester.actionFocus(tf);
        int code = KeyEvent.VK_ESCAPE;
        int mods = 0;
        tester.actionKeyStroke("VK_ESCAPE", "0");
        assertTrue("Never received key press", kw.gotPress);
        assertTrue("Never received release", kw.gotRelease);
        assertEquals("Wrong key code", code, kw.code);
        assertEquals("Wrong modifiers", tester.getKeyModifiers(mods), 
                     tester.getKeyModifiers(kw.modifiers));
    }

    private boolean flagOne = false;
    private boolean flagTwo = false;
    public void testScreenMenuBar() {

        String prop = "com.apple.macos.useScreenMenuBar";
        String old = System.getProperty(prop);
        System.setProperty(prop, "true");
        JMenuBar mb = new JMenuBar();
        JMenu menu = new JMenu("File");
        menu.add(new JMenuItem("One"));
        JMenuItem mi1 = new JMenuItem("Two");
        mi1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) { flagOne = true; }
        });
        menu.add(mi1);
        JMenu sub = new JMenu("Submenu");
        sub.add(new JMenuItem("Sub one"));
        JMenuItem mi2 = new JMenuItem("Sub two");
        mi2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) { flagTwo = true; }
        });
        sub.add(mi2);
        menu.add(sub);
        mb.add(menu);
        JFrame frame = (JFrame)showFrame(new JLabel(getName()));
        frame.setJMenuBar(mb);
        try {
            tester.actionSelectMenuItem(mi1);
            assertTrue("Menu item in menubar menu not hit", flagOne);
            tester.actionSelectMenuItem(mi2);
            assertTrue("Menu item in menubar submenu not hit", flagTwo);
        }
        finally {
            // Not sure if reverting actually has any effect
            if (old != null)
                System.setProperty(prop, old);
            else 
                System.setProperty(prop, "false");
        }
    }

    public OSXTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        TestHelper.runTests(args, OSXTest.class);
    }
}
